function [lambda,trans,y_p,refl] = Nanoslit(lambdamin,lambdamax,nb_p,theta0,nb_diff,nb_modes,d0,h0,a0,n1,n2,n3,em0,cx)

%% Transmission theory for Squarehole array based
% By Andr-Pierre Blanchard-Dionne
% (if you use this material please cite paper entitled :" Sensing With
% Periodic Nanohole Array" published in Advances In Optics and Photonics

clearvars -except lambdamin lambdamax nb_p theta0 nb_diff nb_modes d0 h0 a0 n1 n2 n3 em0 cx                  %Numbers of wavelength points considered
lambda = linspace(lambdamin,lambdamax,nb_p);    %Wavelength in microns

%% Parameters of the method
theta = theta0;                          %incident angle in degrees
%nb_diff                                 %Numbers of diffraction order considered in x

%nb_modes =1;                           %Numbers of TM mode considered. The mode's number is defined later
                                        %Higher numbers than 1 uses Nelder-Mead for beta (time consumming)     
                               
d = d0.*1E-6;                       %Period of the array
h = h0.*1E-6;                       %Thickness of the metal layer
a = a0.*1E-6;                       %Diameter of the hole

e1 = n1.^2;                       %Dielectric Constant of medium 1
e2 = n2.^2;                       %Dielectric constant of medium 2 (inside the hole)
e3 = n3.^2;                       %Dielectric constant of medium 3 (substrate)

switch em0
    case 'Au'
        donnees_au = load('AU_JC2.txt');
        n_AU = spline((donnees_au(:,1)),(donnees_au(:,2)),lambda);
        k_AU = spline((donnees_au(:,1)),(donnees_au(:,3)),lambda);
        nAU = n_AU + j*k_AU;
        em = nAU.^2;                        %Dielectric constant of the metal
        Zs = 1./sqrt(em);
        
    case 'Ag'
        %Ag
        donnees_si = load('Ag.txt');
        n_SI = spline(smooth(donnees_si(:,1)),smooth(donnees_si(:,2)),lambda);
        k_SI = spline(smooth(donnees_si(:,1)),smooth(donnees_si(:,3)),lambda);
        nSI = n_SI + j*k_SI;
        em = nSI.^2;                        %Dielectric constant of the metal
        Zs = 1./sqrt(em);
        
    case 'Al'
        %Al
        donnees_al = load('Al.txt');
        n_Al = spline(smooth(donnees_al(:,1)),smooth(donnees_al(:,2)),lambda);
        k_Al = spline(smooth(donnees_al(:,1)),smooth(donnees_al(:,3)),lambda);
        nAL = n_Al + j*k_Al;        
        em = nAL.^2;                        %Dielectric constant of the metal
        Zs = 1./sqrt(em);
        
    case 'PEC'
        em = 'PEC';
        Zs =0;
end
%% Constants and variables initialization
if theta == 0
m_tm = [0 2 4 6 8];            %Must be even and greater or equal to 2
else
m_tm = [0 1 2 3 4 5 6 7 8];
end

d =d+1e-14;a=a+1e-14;h=h+3e-14;     %to avoid singular matrix when a and d are multiples and lambda

nb_difftot = 2.*nb_diff+1;            
n0 = floor(nb_difftot./2)+1;
c =  2.9979E8; kw = 2*pi./(lambda*1E-6); w = 2*pi*c./(lambda*1E-6);
theta = theta*pi/180;kw1 = kw.*sqrt(e1);kw3 = kw.*sqrt(e3);kr = 2*pi./(d)+1e-2;
kx0 = kw.*sqrt(e1).*sin(theta);theta3 = asin(sqrt(e1).*sin(theta)/sqrt(e3));
kx03 = kw.*sqrt(e3).*sin(theta3);kw2 = kw.*sqrt(e2);inc2 = 0;
k1 = zeros(nb_difftot,length(lambda));k3 = k1;kz1=k1;kz3=k1;
Yp1 =k1;Yp3=k1;Zs2 =k1;beta(nb_modes,:) = zeros(1,length(lambda));
bx(nb_modes,:) = zeros(1,length(lambda));
norm1(nb_modes,:) = zeros(1,length(lambda));Sn12 = cell(1,nb_modes);Sn12_t = Sn12;Sn23 = Sn12;Sn23_t = Sn12;
Y_alpha = Sn12;trans(1,1:length(lambda)) = 0;refl = trans;
for ll = 1:nb_modes
    Sn12{ll} = zeros(1,length(lambda));Sn12_t{ll} = zeros(1,length(lambda));
    Sn23{ll} = zeros(1,length(lambda));Sn23_t{ll} = zeros(1,length(lambda));
end

%% Overlap integral calculation
for pf = 1:nb_modes
    inc2 = inc2+1;
    inc = 0;
    m=m_tm(inc2);
    if isequal(em,'PEC')
        beta(inc2,:) = sqrt(kw2.^2-(m.*pi./a).^2);
        bx(inc2,:) = m.*pi./a+1e-10;
    else
        if m == 0
            %Approximate equation for the waveguide mode of a parralel
            %plate waveguide for the funsamental mode. See Eq. 12 in the Article
            beta(inc2,:) = sqrt(e2).*sqrt(1+lambda.*1E-6./(pi*a*sqrt(-em)).*sqrt(1+e2./(-em))).*kw;
            bx(inc2,:) = sqrt(e2.*kw.^2-beta(inc2,:).^2)+1e-10;
        else
            %Nelder-Mead algorithm for higher modes
            ini_guess = sqrt(kw(1).^2.*e2-((m*pi./a(1)).^2)/1.25);
            x = [real(ini_guess) imag(ini_guess)];
            for l = 1:length(lambda)
                x0 = [x(1) x(2)];
                fun = @(x)abs(tanh(sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*e2).*a/2-m.*1i.*pi/2)+e2.*sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*em(l))./(em(l).*sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*e2)));
                %options = optimset('MaxIter',2000,'MaxFunEvals',2000,'TolFun',1e-5,'TolX',1e-5);
                [x,output] = fminsearch(fun,x0);
                beta(inc2,l) =  x(1)+1i.*x(2);
            end
            bx(inc2,:) = sqrt(e2.*kw.^2-beta(inc2,:).^2)+1;
        end
    end
    %_____________________________________________________________
    %_____________________________________________________________
    %Calculation of overlap integrals and admittance factors
    for  l = -nb_diff:nb_diff
        inc = inc+1;
        k1(inc,:) = kx0+l*kr;
        k3(inc,:) = kx03+l*kr;
        kz1(inc,:) = sqrt(kw1.^2-k1(inc,:).^2);
        kz3(inc,:) = sqrt(kw3.^2-k3(inc,:).^2);
        %Normalization factor
        if mod(m,2) == 0 || 2
            norm1(inc2,:) = ((a.*bx(inc2,:)+sin(a.*bx(inc2,:)))./(2.*bx(inc2,:))).^(-1/2);
        else
            norm1(inc2,:) = ((a.*bx(inc2,:)-sin(a.*bx(inc2,:)))./(2.*bx(inc2,:))).^(-1/2);
        end
        norm2 = 1./sqrt(d);
        
        %Normalized Overlap integral equations. Eq 17-18 in the article
        
        if mod(m,4) == 0
            Sn12{inc2}(inc,:) = norm1(inc2,:).*norm2.*(2.*bx(inc2,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k1(inc,:)./2)-2.*k1(inc,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k1(inc,:)./2))./(bx(inc2,:).^2-k1(inc,:).^2);
            Sn23{inc2}(inc,:) = norm1(inc2,:).*norm2.*(2.*bx(inc2,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k3(inc,:)./2)-2.*k3(inc,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k3(inc,:)./2))./(bx(inc2,:).^2-k3(inc,:).^2);
        elseif mod(m,4) == 1
            Sn12{inc2}(inc,:) = norm1(inc2,:).*norm2.*2.*1i.*(k1(inc,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k1(inc,:)./2)-bx(inc2,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k1(inc,:)./2))./(bx(inc2,:).^2-k1(inc,:).^2);
            Sn23{inc2}(inc,:) = norm1(inc2,:).*norm2.*2.*1i.*(k3(inc,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k3(inc,:)./2)-bx(inc2,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k3(inc,:)./2))./(bx(inc2,:).^2-k3(inc,:).^2);            
        elseif mod(m,4) == 2
            Sn12{inc2}(inc,:) = -norm1(inc2,:).*norm2.*(2.*bx(inc2,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k1(inc,:)./2)-2.*k1(inc,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k1(inc,:)./2))./(bx(inc2,:).^2-k1(inc,:).^2);
            Sn23{inc2}(inc,:) = -norm1(inc2,:).*norm2.*(2.*bx(inc2,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k3(inc,:)./2)-2.*k3(inc,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k3(inc,:)./2))./(bx(inc2,:).^2-k3(inc,:).^2);
        else
            Sn12{inc2}(inc,:) = -norm1(inc2,:).*norm2.*2.*1i.*(k1(inc,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k1(inc,:)./2)-bx(inc2,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k1(inc,:)./2))./(bx(inc2,:).^2-k1(inc,:).^2);
            Sn23{inc2}(inc,:) = -norm1(inc2,:).*norm2.*2.*1i.*(k3(inc,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k3(inc,:)./2)-bx(inc2,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k3(inc,:)./2))./(bx(inc2,:).^2-k3(inc,:).^2);
        end
        Sn12_t{inc2}(inc,:) = conj(Sn12{inc2}(inc,:));
        Sn23_t{inc2}(inc,:) = conj(Sn23{inc2}(inc,:));
        
        %Plane Wave Admittances.
        Yp1(inc,:) = (kw.*e1./kz1(inc,:));
        Yp3(inc,:) = (kw.*e3./kz3(inc,:));
        Zs2(inc,:) = Zs;
    end
    %Guided mode admittance
    Y_alpha{inc2} = kw.*e2./beta(inc2,:);
end

%Coupling Factor G. Eq 25 in the article
for l = 1:nb_modes
    for ll = 1:nb_modes
        G00(l,ll,:) = 1i.*sum(Yp1.*(Sn12{l}.*Sn12_t{ll})./(1+Yp1.*Zs2));
        Gp00(l,ll,:) = 1i.*sum(Yp3.*(Sn23{l}.*Sn23_t{ll})./(1+Yp3.*Zs2));
        Mat1(l,ll,:) = squeeze(G00(l,ll,:)).'.*(1-Zs.*Y_alpha{ll});
    end
    Mat1(l,l,:) = squeeze(Mat1(l,l,:)) + 1i.*Y_alpha{l}.';
    % Use of sqrt(Yp)in order to normalize the energy flux
    I0(l,:) = 2.*1i.*Sn12{l}(n0,:).*sqrt(Yp1(n0,:))./(1+Yp1(n0,:).*Zs);
end

%% Scattering coefficient calculation
%Matrix definition used for the calculation of reflection coefficient 2-3
%and 2-1. See Eq 23 

for l = 1:nb_modes
    for ll = 1:nb_modes
        for lll = 1:nb_modes
            Matrho23{l}(ll,lll,:) = (1-Zs.*Y_alpha{lll}).*squeeze(Gp00(ll,lll,:)).';
            Matrho21{l}(ll,lll,:) = (1-Zs.*Y_alpha{lll}).*squeeze(G00(ll,lll,:)).';
            if ll == l
                B_rho23{l}(ll,:) = 1i.*Y_alpha{ll}-(1+Zs.*Y_alpha{l}).*squeeze(Gp00(ll,l,:)).';
                B_rho21{l}(ll,:) = 1i.*Y_alpha{ll}-(1+Zs.*Y_alpha{l}).*squeeze(G00(ll,l,:)).';
            else
                B_rho23{l}(ll,:) = -(1+Zs.*Y_alpha{l}).*squeeze(Gp00(ll,l,:)).';
                B_rho21{l}(ll,:) = -(1+Zs.*Y_alpha{l}).*squeeze(G00(ll,l,:)).';
            end
        end
        Matrho23{l}(ll,ll,:) = squeeze(Matrho23{l}(ll,ll,:)).'+1i.*Y_alpha{ll};
        Matrho21{l}(ll,ll,:) = squeeze(Matrho21{l}(ll,ll,:)).'+1i.*Y_alpha{ll};
    end
end

inc12 = 0;inc3 = 0;
%Calculation of the scattering coefficient t12 and p23, p21
%See Eq 21,23
for l = 1:length(lambda)
    tau12(:,l) = Mat1(:,:,l)\I0(:,l);
    for ll = 1:nb_modes
        rho23{ll}(:,l) = Matrho23{ll}(:,:,l)\B_rho23{ll}(:,l);
        rho21{ll}(:,l) = Matrho21{ll}(:,:,l)\B_rho21{ll}(:,l);
    end
end
%Calculation of the scattering coefficient p12
%See Eq 22
for ll = 1:nb_difftot
    inc12 = inc12+1;
    somme12 = 0;
    for lll = 1:nb_modes
        somme12 = somme12+Sn12_t{lll}(ll,:).*tau12(lll,:).*(1-Zs.*Y_alpha{lll});
    end
    if ll == n0
        rho12(inc12,:) = (-(1-Zs.*Yp1(n0,:))+somme12)./(1+Zs.*Yp1(ll,:));
    else
        rho12(inc12,:) =somme12./(1+Zs.*Yp1(ll,:)) ;
    end
end
%Calculation of the scattering coefficient t23,t21
%See Eq 24
for l = 1:nb_modes
    for ll = 1:nb_difftot
        somme =0;
        somme2 = 0;
        for lll = 1:nb_modes
            somme =somme +rho23{l}(lll,:).*Sn23_t{lll}(ll,:).*(1-Zs.*Y_alpha{lll});
            somme2 =somme2 +rho21{l}(lll,:).*Sn12_t{lll}(ll,:).*(1-Zs.*Y_alpha{lll});
        end
        tau23{l}(ll,:) = (Sn23_t{l}(ll,:).*(1+Zs.*Y_alpha{l})+somme)./(1+Zs.*Yp3(ll,:));
        tau21{l}(ll,:) = (Sn12_t{l}(ll,:).*(1+Zs.*Y_alpha{l})+somme2)./(1+Zs.*Yp1(ll,:));
    end
    %Normalization of the energy flow, output
    tau23{l}(n0,:) = tau23{l}(n0,:).*sqrt(Yp3(n0,:));
end

inc = 0;
%% Multiple path calculation

B = zeros(nb_modes,length(lambda));
A = zeros(nb_modes,length(lambda));
%Calculation of the transmission factor t
%See Eqs. 28-31
for l = 1:nb_modes
    inc1 = 0;
    for ll = 1:nb_modes
        for lll = 1:nb_modes
            inc2 = 0;
            inc1 = inc1+1;
            vec(:,inc1) = rho23{l}(lll,:).*exp(1i.*beta(lll,:).*h).*rho21{lll}(ll,:).*exp(1i.*beta(ll,:).*h);
            t(inc1,:) = tau23{ll}(n0,:)./tau23{l}(n0,:);
            for iv = 1:nb_modes
                for v = 1:nb_modes
                    inc2 = inc2+1;
                    Mat(inc1,inc2,:) = rho23{ll}(v,:).*exp(1i.*beta(v,:).*h).*rho21{v}(iv,:).*exp(1i.*beta(iv,:).*h);
                end
            end
        end
    end
    for lam = 1:length(lambda)
        cf(lam) = vec(lam,:)/(eye(nb_modes.^2)-Mat(:,:,lam))*t(:,lam);
    end
    trans = trans+tau12(l,:).*exp(1i.*beta(l,:).*h).*tau23{l}(n0,:).*(1+cf);
end

clear t cf cf2 t2
%Calculation of the reflection coefficient and modal amplitude B.
%See eq 32,34
for l = 1:nb_modes
    for ll = 1:nb_modes
        inc1 = 0;
        for lll = 1:nb_modes
            for iv = 1:nb_modes
                inc1 = inc1+1;
                vec(:,inc1) = rho23{l}(lll,:).*exp(1i.*beta(lll,:).*h).*rho21{lll}(iv,:).*exp(1i.*beta(iv,:).*h);
                inc2 = 0;
                t(inc1,1:length(lambda)) = rho23{iv}(ll,:)./rho23{l}(ll,:);
                for v = 1:nb_modes
                    for vi = 1:nb_modes
                        inc2 = inc2+1;
                        Mat(inc1,inc2,:) = rho23{iv}(v,:).*exp(1i.*beta(v,:).*h).*rho21{v}(vi,:).*exp(1i.*beta(vi,:).*h);
                    end
                end
            end
        end
        for lam = 1:length(lambda)
            cf(lam) = vec(lam,:)/(eye(nb_modes.^2)-Mat(:,:,lam))*t(:,lam);
        end
        refl = refl+tau12(l,:).*exp(1i.*beta(l,:).*h).*rho23{l}(ll,:).*exp(1i.*beta(ll,:).*h).*tau21{ll}(n0,:).*(1+cf);
    end
end

refl = refl+rho12(n0,:);
[x_p] = findpeaks(abs(trans).^2);
if isempty(x_p) ==1
    y_p=NaN;
else
    [x_p,y_p] = findpeaks(abs(trans).^2);
    y_p = y_p(end+1-cx);
end
if length(find(imag(beta)<0))>1
    fprintf(2,'Warning, imag(beta) is negative, fminsearch wasn''t effective. Use a smaller lambda as first value \n')
end