function [x,z,E,lambda,transn0,refln0,y_p] = Nanoslit_field(lambdamin,lambdamax,nb_p,theta0,nb_diff,nb_modes,d0,h0,a0,n1,n2,n3,em0,cx,domain,res,y_p)


%% Transmission theory for Squarehole array based
% By Andr-Pierre Blanchard-Dionne
% (if you use this material please cite paper entitled :" Sensing With
% Periodic Nanohole Array" published in Advances In Optics and Photonics

clearvars -except lambdamin lambdamax nb_p theta0 nb_diff nb_modes d0 h0 a0 n1 n2 n3 em0 cx domain res y_p             %Numbers of wavelength points considered
lambda = linspace(lambdamin,lambdamax,nb_p);    %Wavelength in microns

%% Parameters of the method
theta = theta0;                          %incident angle in degrees
%nb_diff                       %Numbers of diffraction order considered in x


%% Parameters of the method
theta = theta0;                          %incident angle in degrees
%nb_diff =nb_diff;                        %Numbers of diffraction order considered in x

%nb_modes =2;                        %Numbers of TM mode considered. The mode's number is defined later
                                    %Higher numbers than 1 uses Nelder-Mead for beta (time consumming)     
                               
d = d0.*1E-6;                       %Period of the array
h = h0.*1E-6;                       %Thickness of the metal layer
a = a0.*1E-6;                       %Diameter of the hole

e1 = n1.^2;                       %Dielectric Constant of medium 1
e2 = n2.^2;                       %Dielectric constant of medium 2 (inside the hole)
e3 = n3.^2;                       %Dielectric constant of medium 3 (substrate)

switch em0
    case 'Au'
        donnees_au = load('AU_JC2.txt');
        n_AU = spline((donnees_au(:,1)),(donnees_au(:,2)),lambda);
        k_AU = spline((donnees_au(:,1)),(donnees_au(:,3)),lambda);
        nAU = n_AU + j*k_AU;
        em = nAU.^2;                        %Dielectric constant of the metal
        Zs = 1./sqrt(em);
        
    case 'Ag'
        %Ag
        donnees_si = load('Ag.txt');
        n_SI = spline(smooth(donnees_si(:,1)),smooth(donnees_si(:,2)),lambda);
        k_SI = spline(smooth(donnees_si(:,1)),smooth(donnees_si(:,3)),lambda);
        nSI = n_SI + j*k_SI;
        em = nSI.^2;                        %Dielectric constant of the metal
        Zs = 1./sqrt(em);
        
    case 'Al'
        %Al
        donnees_al = load('Al.txt');
        n_Al = spline(smooth(donnees_al(:,1)),smooth(donnees_al(:,2)),lambda);
        k_Al = spline(smooth(donnees_al(:,1)),smooth(donnees_al(:,3)),lambda);
        nAL = n_Al + j*k_Al;        
        em = nAL.^2;                        %Dielectric constant of the metal
        Zs = 1./sqrt(em);
        
    case 'PEC'
        em = 'PEC';
        Zs =0;
end
                    

%% Parameters of the field plots

dom = domain.*1E-6;                       %Size of the domain (Region 1 and region 3)
step = res.*1E-9;                        %Resolution of the field evaluation


%% Constants and variables initialization
if theta == 0
m_tm = [0 2 4 6 8];            %Must be even and greater or equal to 2
else
m_tm = [0 1 2 3 4 5 6 7 8];
end

d =d+1e-14;a=a+1e-14;h=h+3e-14;     %to avoid singular matrix when a and d are multiples and lambda

nb_difftot = 2.*nb_diff+1;            
n0 = floor(nb_difftot./2)+1;
c =  2.9979E8; kw = 2*pi./(lambda*1E-6); w = 2*pi*c./(lambda*1E-6);
theta = theta*pi/180;kw1 = kw.*sqrt(e1);kw3 = kw.*sqrt(e3);kr = 2*pi./(d);
kx0 = kw.*sqrt(e1).*sin(theta);ky0(1:length(lambda)) = 0;theta3 = asin(sqrt(e1).*sin(theta)/sqrt(e3));
kx03 = kw.*sqrt(e3).*sin(theta3);kw2 = kw.*sqrt(e2);inc2 = 0;inc1 = 0;
k1 = zeros(nb_difftot,length(lambda));k3 = k1;kz1=k1;kz3=k1;
Yp1 =k1;Yp3=k1;Zs2 =k1;beta(nb_modes,:) = zeros(1,length(lambda));
bx(nb_modes,:) = zeros(1,length(lambda));e_eff = bx;beta1 = bx;
norm1(nb_modes,:) = zeros(1,length(lambda));Sn12 = cell(1,nb_modes);Sn12_t = Sn12;Sn23 = Sn12;Sn23_t = Sn12;
Y_alpha = Sn12;trans=zeros(nb_difftot,length(lambda));refl = trans;
for ll = 1:nb_modes
    Sn12{ll} = zeros(1,length(lambda));Sn12_t{ll} = zeros(1,length(lambda));
    Sn23{ll} = zeros(1,length(lambda));Sn23_t{ll} = zeros(1,length(lambda));
end
d = d+1e-12;a= a+2e-12;h =  h+3e-12;


%% Overlap integral calculation
for pf = 1:nb_modes
    inc2 = inc2+1;
    inc = 0;
    m=m_tm(inc2);
    if isequal(em,'PEC')
        beta(inc2,:) = sqrt(kw2.^2-(m.*pi./a).^2);
        bx(inc2,:) = m.*pi./a+1e-10;
    else
        if m == 0
            %Approximate equation for the waveguide mode of a parralel
            %plate waveguide for the funsamental mode. See Eq. 12 in the Article
            beta(inc2,:) = sqrt(e2).*sqrt(1+lambda.*1E-6./(pi*a*sqrt(-em)).*sqrt(1+e2./(-em))).*kw;
            bx(inc2,:) = sqrt(e2.*kw.^2-beta(inc2,:).^2)+1e-10;
        else
            %Nelder-Mead algorithm for higher modes
            ini_guess = sqrt(kw(1).^2.*e2-((m*pi./a(1)).^2)/1.25);
            x = [real(ini_guess) imag(ini_guess)];
            for l = 1:length(lambda)
                x0 = [x(1) x(2)];
                fun = @(x)abs(tanh(sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*e2).*a/2-m.*1i.*pi/2)+e2.*sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*em(l))./(em(l).*sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*e2)));
                options = optimset('MaxIter',2000,'MaxFunEvals',2000,'TolFun',1e-5,'TolX',1e-5);
                [x,output] = fminsearch(fun,x0);
                beta(inc2,l) =  x(1)+1i.*x(2);
            end
            bx(inc2,:) = sqrt(e2.*kw.^2-beta(inc2,:).^2)+1;
        end
    end
    %_____________________________________________________________
    %_____________________________________________________________
    %Calculation of overlap integrals and admittance factors
    %Normalization factor
    
    if mod(m,2) == 0 || 2
        norm1(inc2,:) = ((a.*bx(inc2,:)+sin(a.*bx(inc2,:)))./(2.*bx(inc2,:))).^(-1/2);
    else
        norm1(inc2,:) = ((a.*bx(inc2,:)-sin(a.*bx(inc2,:)))./(2.*bx(inc2,:))).^(-1/2);
    end
    
    norm2 = 1./sqrt(d);
    
    for  l = -nb_diff:nb_diff
        inc = inc+1;
        k1(inc,:) = kx0+l*kr;
        k3(inc,:) = kx03+l*kr;
        kz1(inc,:) = sqrt(kw1.^2-k1(inc,:).^2);
        kz3(inc,:) = sqrt(kw3.^2-k3(inc,:).^2);
        
        %Normalized Overlap integral equations. Eq 17-18 in the article
        
        if mod(m,4) == 0
            Sn12{inc2}(inc,:) = norm1(inc2,:).*norm2.*(2.*bx(inc2,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k1(inc,:)./2)-2.*k1(inc,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k1(inc,:)./2))./(bx(inc2,:).^2-k1(inc,:).^2);
            Sn23{inc2}(inc,:) = norm1(inc2,:).*norm2.*(2.*bx(inc2,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k3(inc,:)./2)-2.*k3(inc,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k3(inc,:)./2))./(bx(inc2,:).^2-k3(inc,:).^2);
        elseif mod(m,4) == 1
            Sn12{inc2}(inc,:) = norm1(inc2,:).*norm2.*2.*1i.*(k1(inc,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k1(inc,:)./2)-bx(inc2,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k1(inc,:)./2))./(bx(inc2,:).^2-k1(inc,:).^2);
            Sn23{inc2}(inc,:) = norm1(inc2,:).*norm2.*2.*1i.*(k3(inc,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k3(inc,:)./2)-bx(inc2,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k3(inc,:)./2))./(bx(inc2,:).^2-k3(inc,:).^2);            
        elseif mod(m,4) == 2
            Sn12{inc2}(inc,:) = -norm1(inc2,:).*norm2.*(2.*bx(inc2,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k1(inc,:)./2)-2.*k1(inc,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k1(inc,:)./2))./(bx(inc2,:).^2-k1(inc,:).^2);
            Sn23{inc2}(inc,:) = -norm1(inc2,:).*norm2.*(2.*bx(inc2,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k3(inc,:)./2)-2.*k3(inc,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k3(inc,:)./2))./(bx(inc2,:).^2-k3(inc,:).^2);
        else
            Sn12{inc2}(inc,:) = -norm1(inc2,:).*norm2.*2.*1i.*(k1(inc,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k1(inc,:)./2)-bx(inc2,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k1(inc,:)./2))./(bx(inc2,:).^2-k1(inc,:).^2);
            Sn23{inc2}(inc,:) = -norm1(inc2,:).*norm2.*2.*1i.*(k3(inc,:).*sin(a.*bx(inc2,:)./2).*cos(a.*k3(inc,:)./2)-bx(inc2,:).*cos(a.*bx(inc2,:)./2).*sin(a.*k3(inc,:)./2))./(bx(inc2,:).^2-k3(inc,:).^2);
        end
        
        Sn12_t{inc2}(inc,:) = conj(Sn12{inc2}(inc,:));
        Sn23_t{inc2}(inc,:) = conj(Sn23{inc2}(inc,:));
        
        %Plane Wave Admittances. 
        Yp1(inc,:) = (kw.*e1./kz1(inc,:));
        Yp3(inc,:) = (kw.*e3./kz3(inc,:));
        Zs2(inc,:) = Zs;
    end
        %Guided mode admittance
        Y_alpha{inc2} = kw.*e2./beta(inc2,:);
end

%Coupling Factor G. Eq 25 in the article
for l = 1:nb_modes
    for ll = 1:nb_modes
        G00(l,ll,:) = 1i.*sum(Yp1.*(Sn12{l}.*Sn12_t{ll})./(1+Yp1.*Zs2));
        Gp00(l,ll,:) = 1i.*sum(Yp3.*(Sn23{l}.*Sn23_t{ll})./(1+Yp3.*Zs2));
        Mat1(l,ll,:) = squeeze(G00(l,ll,:)).'.*(1-Zs.*Y_alpha{ll});
    end
    Mat1(l,l,:) = squeeze(Mat1(l,l,:)) + 1i.*Y_alpha{l}.';
    % Use of sqrt(Yp)in order to normalize the energy flux
    I0(l,:) = 2.*1i.*Sn12{l}(n0,:).*sqrt(Yp1(n0,:))./(1+Yp1(n0,:).*Zs);
end

%% Scattering coefficient calculation
%Matrix definition used for the calculation of reflection coefficient 2-3
%and 2-1. See Eq 23 
for l = 1:nb_modes
    for ll = 1:nb_modes
        for lll = 1:nb_modes
            Matrho23{l}(ll,lll,:) = (1-Zs.*Y_alpha{lll}).*squeeze(Gp00(ll,lll,:)).';
            Matrho21{l}(ll,lll,:) = (1-Zs.*Y_alpha{lll}).*squeeze(G00(ll,lll,:)).';
            if ll == l
                B_rho23{l}(ll,:) = 1i.*Y_alpha{ll}-(1+Zs.*Y_alpha{l}).*squeeze(Gp00(ll,l,:)).';
                B_rho21{l}(ll,:) = 1i.*Y_alpha{ll}-(1+Zs.*Y_alpha{l}).*squeeze(G00(ll,l,:)).';
            else
                B_rho23{l}(ll,:) = -(1+Zs.*Y_alpha{l}).*squeeze(Gp00(ll,l,:)).';
                B_rho21{l}(ll,:) = -(1+Zs.*Y_alpha{l}).*squeeze(G00(ll,l,:)).';
            end
        end
        Matrho23{l}(ll,ll,:) = squeeze(Matrho23{l}(ll,ll,:)).'+1i.*Y_alpha{ll};
        Matrho21{l}(ll,ll,:) = squeeze(Matrho21{l}(ll,ll,:)).'+1i.*Y_alpha{ll};
    end
end

inc12 = 0;inc3 = 0;
%Calculation of the scattering coefficient t12 and p23, p21
%See Eq 21,23
for l = 1:length(lambda)
    tau12(:,l) = Mat1(:,:,l)\I0(:,l);
    for ll = 1:nb_modes
        rho23{ll}(:,l) = Matrho23{ll}(:,:,l)\B_rho23{ll}(:,l);
        rho21{ll}(:,l) = Matrho21{ll}(:,:,l)\B_rho21{ll}(:,l);
    end
end
%Calculation of the scattering coefficient p12
%See Eq 22
for ll = 1:nb_difftot
    inc12 = inc12+1;
    somme12 = 0;
    for lll = 1:nb_modes
        somme12 = somme12+Sn12_t{lll}(ll,:).*tau12(lll,:).*(1-Zs.*Y_alpha{lll});
    end
    if ll == n0
        rho12(inc12,:) = (-(1-Zs.*Yp1(n0,:))+somme12)./(1+Zs.*Yp1(ll,:));
    else
        rho12(inc12,:) =somme12./(1+Zs.*Yp1(ll,:)) ;
    end
end
%Calculation of the scattering coefficient t23,t21
%See Eq 24
for l = 1:nb_modes
    for ll = 1:nb_difftot
        somme =0;
        somme2 = 0;
        for lll = 1:nb_modes
            somme =somme +rho23{l}(lll,:).*Sn23_t{lll}(ll,:).*(1-Zs.*Y_alpha{lll});
            somme2 =somme2 +rho21{l}(lll,:).*Sn12_t{lll}(ll,:).*(1-Zs.*Y_alpha{lll});
        end
        tau23{l}(ll,:) = (Sn23_t{l}(ll,:).*(1+Zs.*Y_alpha{l})+somme)./(1+Zs.*Yp3(ll,:));
        tau21{l}(ll,:) = (Sn12_t{l}(ll,:).*(1+Zs.*Y_alpha{l})+somme2)./(1+Zs.*Yp1(ll,:));
    end
    %Normalization of the energy flow, output
    tau23{l}(n0,:) = tau23{l}(n0,:).*sqrt(Yp3(n0,:));
end


%% Multiple path calculation
B = zeros(nb_modes,length(lambda));
A = zeros(nb_modes,1);

%Calculation of the transmission factor t
%See Eqs. 28-31
for l = 1:nb_modes
    inc1 = 0;
    for ll = 1:nb_modes
        for lll = 1:nb_modes
            inc2 = 0;
            inc1 = inc1+1;
            vec(:,inc1) = rho23{l}(lll,:).*exp(1i.*beta(lll,:).*h).*rho21{lll}(ll,:).*exp(1i.*beta(ll,:).*h);
            for vv = 1:nb_difftot 
                t(inc1,:,vv) = tau23{ll}(vv,:)./tau23{l}(vv,:);
            end
            for iv = 1:nb_modes
                for v = 1:nb_modes
                    inc2 = inc2+1;
                    Mat(inc1,inc2,:) = rho23{ll}(v,:).*exp(1i.*beta(v,:).*h).*rho21{v}(iv,:).*exp(1i.*beta(iv,:).*h);
                end
            end
        end
    end
    
    for lam = 1:length(lambda)
        for vv = 1:nb_difftot
            cf(lam,vv) = vec(lam,:)/(eye(nb_modes.^2)-Mat(:,:,lam))*t(:,lam,vv);
        end
    end
    for vv = 1:nb_difftot
        trans(vv,:) = trans(vv,:)+tau12(l,:).*exp(1i.*beta(l,:).*h).*tau23{l}(vv,:).*(1+cf(:,vv).');
    end
end

clear t cf
t=zeros(nb_modes,length(lambda));cf=zeros(1,length(lambda));

%Calculation of the reflection coefficient and modal amplitude B.
%See eq 32,34
for l = 1:nb_modes
    for ll = 1:nb_modes
        inc1 = 0;
        for lll = 1:nb_modes
            for iv = 1:nb_modes
                inc1 = inc1+1;
                vec(:,inc1) = rho23{l}(lll,:).*exp(1i.*beta(lll,:).*h).*rho21{lll}(iv,:).*exp(1i.*beta(iv,:).*h);
                inc2 = 0;
                t(inc1,1:length(lambda)) = rho23{iv}(ll,:)./rho23{l}(ll,:);
                for v = 1:nb_modes
                    for vi = 1:nb_modes
                        inc2 = inc2+1;
                        Mat(inc1,inc2,:) = rho23{iv}(v,:).*exp(1i.*beta(v,:).*h).*rho21{v}(vi,:).*exp(1i.*beta(vi,:).*h);
                    end
                end
            end
        end
        for lam = 1:length(lambda)
            cf(lam) = vec(lam,:)/(eye(nb_modes.^2)-Mat(:,:,lam))*t(:,lam);
        end
        for vv = 1:nb_difftot
            refl(vv,:) = refl(vv,:)+tau12(l,:).*exp(1i.*beta(l,:).*h).*rho23{l}(ll,:).*exp(1i.*beta(ll,:).*h).*tau21{ll}(vv,:).*(1+cf);
        end
        B(ll,:) = B(ll,:)+ tau12(l,:).*exp(1i.*h.*beta(l,:)).*rho23{l}(ll,:).*exp(1i.*h.*beta(ll,:)./2).*(1+cf);
    end
end

refl = refl+rho12;
if cx > 0
[x_p,y_p] = findpeaks(abs(trans(n0,:)).^2);
y_p = y_p(end+1-cx);
end

vec2=complex(1,nb_modes);cf2=complex(1);
Mat2=zeros(nb_modes,nb_modes,1);t2=complex(nb_modes,1);

%Calculation of Modal Amplitude A
%See Eq. 34
for l = 1:nb_modes
    somme_A =0;
    for ll = 1:nb_modes
        for lll = 1:nb_modes
            inc2 = 0;
            for iv = 1:nb_modes
                for v = 1:nb_modes
                    inc2 = inc2+1;
                    inc3 = 0;
                    vec2(1,inc2) = rho23{ll}(iv,y_p).*exp(1i.*beta(iv,y_p).*h).*rho21{iv}(v,y_p).*exp(1i.*beta(v,y_p).*h);
                    for vi = 1:nb_modes
                        for vii = 1:nb_modes
                            inc3 = inc3+1;
                            Mat2(inc2,inc3,1) = rho23{v}(vi,y_p).*exp(1i.*beta(vi,y_p).*h).*rho21{vi}(vii,y_p).*exp(1i.*beta(vii,y_p).*h);
                            t2(inc3,1) = rho23{vii}(lll,y_p)./rho23{ll}(lll,y_p);
                        end
                    end
                end
            end
            for lam = 1:length(y_p)
                cf2(lam) =vec2(1,:)/(eye(nb_modes.^2)-Mat2(:,:,1))*t2(:,1);
            end
            somme_A = somme_A + tau12(ll,y_p)./tau12(l,y_p).*exp(1i.*beta(ll,y_p).*h).*rho23{ll}(lll,y_p).*exp(1i.*beta(lll,y_p).*h).*rho21{lll}(l,y_p).*(1+cf2);
        end
    end
    A(l) = tau12(l,y_p).*exp(1i.*beta(l,y_p).*h/2).*(1+somme_A);
end

%%
%Field Expression
nb_plane = 1;
clims = [0 0];

for pp = 1:nb_plane
    x = -d/2:step:d/2;
    z = -dom:step:dom;
    inc4 = 0;
    for l = 1:length(x)
        if x(l) > -a/2 && x(l) < a/2
            start = l;
            inc4 = inc4+1;
        end
    end
    xspread = start-inc4+1:start;
    yspread = 1;
    inc = 0;
    inc1 = 0;inc2 = 0;inc3 = 0;inc4 = 0;
    
    for l = 1:length(z)
        if z(l)<-h/2
            inc1 = inc1+1;
        elseif z(l)<=h/2
            inc2 = inc2+1;
        else
            inc3 = inc3+1;
        end
    end
    
    z1 = z(1:inc1);
    z2 = z(inc1+1:inc1+inc2);
    z3 = z(inc1+1+inc2:inc1++inc2+inc3);
    
    x2 = x(xspread);
    [X1,Z1] = meshgrid(x,z1);
    [X2,Z2] = meshgrid(x2,z2);
    [X3,Z3] = meshgrid(x,z3);
    [X4,Z4] = meshgrid(x(1:xspread(1)-1),z2);
    [X5,Z5] = meshgrid(x(xspread(end)+1:length(x)),z2);
    %Hy = complex(zeros(nb_difftot*2,1,length(z),length(x),length(y)));
    Ex = complex(zeros(nb_diff,1,length(z),length(x)));
    Ez = complex(zeros(nb_diff,1,length(z),length(x)));
    
    for ll = y_p;
        inc_lam = 1;
        for l = 1:nb_difftot;
            if l == n0
                %Hy(l,inc_lam,1:inc1,:) = 1./sqrt(d).*((Yp(l,ll)).*exp(1i.*k(l,ll).*(X1)).*exp(1i.*kz(l,ll).*(Z1+h/2))+ refl(l,ll).*-Yp(l,ll).*exp(1i.*k(l,ll).*(X1)).*exp(-1i.*kz(l,ll).*(Z1+h/2)));
                %Hy(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:) = 1./sqrt(d).*((Yp3(l,ll)).*trans(l,ll).*exp(1i.*k(l,ll).*(X3)).*exp(1i.*kz(l,ll).*(Z3-h/2)));
                Ex(l,inc_lam,1:inc1,:) = 1./sqrt(d).*(exp(1i.*k1(l,ll).*(X1)).*exp(1i.*kz1(l,ll).*(Z1+h/2))+ refl(l,ll).*exp(1i.*k1(l,ll).*(X1)).*exp(-1i.*kz1(l,ll).*(Z1+h/2+step/2)));
                Ex(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:) = 1./sqrt(d).*(trans(l,ll).*exp(1i.*k3(l,ll).*(X3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
                Ez(l,inc_lam,1:inc1,:) = -k1(l,ll)./kz1(l,ll)./sqrt(d).*(exp(1i.*k1(l,ll).*(X1)).*exp(1i.*kz1(l,ll).*(Z1+h/2))-refl(l,ll).*exp(1i.*k1(l,ll).*(X1)).*exp(-1i.*kz1(l,ll).*(Z1+h/2+step/2)));
                Ez(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:) = k3(l,ll)./kz3(l,ll)./sqrt(d).*(trans(l,ll).*exp(1i.*k3(l,ll).*(X3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
            else
                %Hy(l,inc_lam,1:inc1,:) = 1./sqrt(d).*(-Yp(l,ll).*refl(l,ll).*exp(1i.*k(l,ll).*(X1)).*exp(-1i.*kz(l,ll).*(Z1+h/2)));
                %Hy(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:) = 1./sqrt(d).*(Yp3(l,ll).*trans(l,ll).*exp(1i.*k(l,ll).*(X3)).*exp(1i.*kz(l,ll).*(Z3-h/2)));
                Ex(l,inc_lam,1:inc1,:) = 1./sqrt(d).*(refl(l,ll).*exp(1i.*k1(l,ll).*(X1)).*exp(-1i.*kz1(l,ll).*(Z1+h/2+step/2)));
                Ex(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:) = 1./sqrt(d).*(trans(l,ll).*exp(1i.*k3(l,ll).*(X3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
                Ez(l,inc_lam,1:inc1,:) = k1(l,ll)./kz1(l,ll)./sqrt(d).*(refl(l,ll).*exp(1i.*k1(l,ll).*(X1)).*exp(-1i.*kz1(l,ll).*(Z1+h/2+step/2)));
                Ez(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:) = k3(l,ll)./kz3(l,ll)./sqrt(d).*(trans(l,ll).*exp(1i.*k3(l,ll).*(X3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
            end
        end
        for l = 1:nb_modes
            %Hy(l,inc_lam,inc1+1:inc1+inc2,start-inc4+1:start) = 1.*(A(l,ll).*exp(1i.*beta(l,ll).*Z2)-B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(Y_alpha1{l}(ll)./sqrt(a).*cos((l-1)*pi/a.*(X2+a/2)));
            if mod(m_tm(l),4) ==0
                Ex(l,inc_lam,inc1+1:inc1+inc2,xspread) = (A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1(l,ll).*cos(bx(l,ll).*X2));
                Ez(l,inc_lam,inc1+1:inc1+inc2,xspread) = 1i.*bx(l,ll)./beta(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1(l,ll).*sin(bx(l,ll).*X2));
               %Ex(l,inc_lam,inc1+1:inc1+inc2,1:xspread(1)-1) = (A(l,ll).*exp(1i.*beta(l,ll).*Z4)+B(l,ll).*exp(-1i.*beta(l,ll).*Z4)).*(norm1(l,ll).*exp(-1i.*alphax(l,ll).*X4));
                %Ex(l,inc_lam,inc1+1:inc1+inc2,xspread(end)+1:length(x)) = (A(l,ll).*exp(1i.*beta(l,ll).*Z5)+B(l,ll).*exp(-1i.*beta(l,ll).*Z5)).*(norm1(l,ll).*exp(1i.*alphax(l,ll).*X5));
            elseif mod(m_tm(l),4) ==1
                Ex(l,inc_lam,inc1+1:inc1+inc2,xspread) = (A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1(l,ll).*sin(bx(l,ll).*X2));    
                Ez(l,inc_lam,inc1+1:inc1+inc2,xspread) = -1i.*bx(l,ll)./beta(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1(l,ll).*cos(bx(l,ll).*X2));
            elseif mod(m_tm(l),4) ==2
                Ex(l,inc_lam,inc1+1:inc1+inc2,xspread) = -(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1(l,ll).*cos(bx(l,ll).*X2));
                Ez(l,inc_lam,inc1+1:inc1+inc2,xspread)= -1i.*bx(l,ll)./beta(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1(l,ll).*sin(bx(l,ll).*X2));
                %Ex(l,inc_lam,inc1+1:inc1+inc2,1:xspread(1)-1) = (A(l,ll).*exp(1i.*beta(l,ll).*Z4)+B(l,ll).*exp(-1i.*beta(l,ll).*Z4)).*(norm1(l,ll).*exp(-1i.*alphax(l,ll).*X4));
                %Ex(l,inc_lam,inc1+1:inc1+inc2,xspread(end)+1:length(x)) = (A(l,ll).*exp(1i.*beta(l,ll).*Z5)+B(l,ll).*exp(-1i.*beta(l,ll).*Z5)).*(norm1(l,ll).*exp(1i.*alphax(l,ll).*X5));
            else
               Ex(l,inc_lam,inc1+1:inc1+inc2,xspread) = -(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1(l,ll).*sin(bx(l,ll).*X2));
               Ez(l,inc_lam,inc1+1:inc1+inc2,xspread) = 1i.*bx(l,ll)./beta(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1(l,ll).*cos(bx(l,ll).*X2));
            end
        end
    end
end

E = (squeeze(abs(sum(Ex(:,1,:,:),1)).^2)+squeeze(abs(sum(Ez(:,1,:,:),1)).^2));%+squeeze(abs(sum(Ez(:,1,:,:,:),1)));
transn0=trans(n0,:);refln0=refl(n0,:);
if length(find(imag(beta)<0))>1
    fprintf(2,'Warning, imag(beta) is negative, fminsearch wasn''t effective. Use a smaller lambda as first value \n')
end