function [absc,ord,E,lambda,transn0,refln0,y_p] = Squarehole_field(lambdamin,lambdamax,nb_p,theta0,nb_diffx,nb_diffy,nb_modes_TM,nb_modes_TE,d0,h0,a0,n1,n2,n3,em0,cx,pp,domain,res,coor,y_p)

%% Transmission theory for Squarehole array based
% By Andr-Pierre Blanchard-Dionne
% (if you use this material please cite paper entitled :" Sensing With
% Periodic Nanohole Array" published in Advances In Optics and Photonics

clearvars -except lambdamin lambdamax nb_p theta0 nb_diffx nb_diffy nb_modes_TE nb_modes_TM d0 h0 a0 n1 n2 n3 em0 cx pp coor domain res y_p           %Numbers of wavelength points considered
lambda = linspace(lambdamin,lambdamax,nb_p);    %Wavelength in microns

%% Parameters of the method
theta = theta0;                          %incident angle in degrees
%nb_diff =nb_diff;                        %Numbers of diffraction order considered in x

%nb_modes =2;                        %Numbers of TM mode considered. The mode's number is defined later
                                    %Higher numbers than 1 uses Nelder-Mead for beta (time consumming)     
d = d0.*1E-6;                       %Period of the array
h = h0.*1E-6;                       %Thickness of the metal layer
a = a0.*1E-6;                       %Diameter of the hole

e1 = n1.^2;                       %Dielectric Constant of medium 1
e2 = n2.^2;                       %Dielectric constant of medium 2 (inside the hole)
e3 = n3.^2;                       %Dielectric constant of medium 3 (substrate)

switch em0
    case 'Au'
        donnees_au = load('AU_JC2.txt');
        n_AU = spline((donnees_au(:,1)),(donnees_au(:,2)),lambda);
        k_AU = spline((donnees_au(:,1)),(donnees_au(:,3)),lambda);
        nAU = n_AU + j*k_AU;
        em = nAU.^2;                        %Dielectric constant of the metal
        Zs = 1./sqrt(em);
        
    case 'Ag'
        %Ag
        donnees_si = load('Ag.txt');
        n_SI = spline(smooth(donnees_si(:,1)),smooth(donnees_si(:,2)),lambda);
        k_SI = spline(smooth(donnees_si(:,1)),smooth(donnees_si(:,3)),lambda);
        nSI = n_SI + j*k_SI;
        em = nSI.^2;                        %Dielectric constant of the metal
        Zs = 1./sqrt(em);
        
    case 'Al'
        %Al
        donnees_al = load('Al.txt');
        n_Al = spline(smooth(donnees_al(:,1)),smooth(donnees_al(:,2)),lambda);
        k_Al = spline(smooth(donnees_al(:,1)),smooth(donnees_al(:,3)),lambda);
        nAL = n_Al + j*k_Al;        
        em = nAL.^2;                        %Dielectric constant of the metal
        Zs = 1./sqrt(em);
        
    case 'PEC'
        em = 'PEC';
        Zs =0;
end


%% Parameters of the field plots

dom = domain.*1E-6;                       %Size of the domain (Region 1 and region 3)
step = res.*1e-9;                        %Resolution of the field evaluation
plane = coor.*1e-6;                  %Coordinate of the plane (must be in between [-dom, dom] for z plane, and [-d d] for x or y plane)

%% Constants and variables initialization
if theta ==0
    mx_te = [0 2 0 2 4 0 4];        %Must be even
    my_te = [1 1 3 3 1 5 3];        %Must be odd and greater or equal to 1
    mx_tm = [2 2 4 0 4];            %Must be even and greater or equal to 2
    my_tm = [1 3 1 5 3];            %Must be odd and greater or equal to 1
else
    mx_te = [0 1 2 0 1 2 2];       %
    my_te = [1 1 1 2 2 1 2];        %Must greater or equal to 1
    mx_tm = [1 2 1 2 3 2 1 4];            %Must greater or equal to 1
    my_tm = [1 1 2 2 2 3 4 1];            %Must greater or equal to 1
end
d = d+1e-14;a=a+2e-14;h=h+3e-14;
c =  2.9979E8; kw = 2*pi./(lambda*1E-6); w = 2*pi*c./(lambda*1E-6);
theta = theta*pi/180;nb_difftot = (nb_diffx.*2+1).*(nb_diffy.*2+1);
n0S = floor(nb_difftot./2)+1;
nb_difftot2=2.*nb_difftot;
trans(nb_difftot2,1:length(lambda)) = 0;refl = trans;
kw1 = kw.*sqrt(e1);kw3 = kw.*sqrt(e3);kr = 2*pi./(d);
kx0 = kw.*sqrt(e1).*sin(theta);ky0(1:length(lambda)) = 0;
theta3 = asin(sqrt(e1).*sin(theta)/sqrt(e3));
kx03 = kw.*sqrt(e3).*sin(theta3);ky03(1:length(lambda)) = 0;
kw2 = kw.*sqrt(e2);
inc2 = 0;
nb_difftot2 = nb_difftot*2;
nb_modes =nb_modes_TE+nb_modes_TM;
kx = zeros(nb_difftot2,length(lambda));kx3 = kx;ky=kx;ky3=kx;kll=kx;kll3=kx;kz=kx;kz3=kx;
fx1=kx;fx3=kx;fy1=kx;fy3=kx;
Yp =kx;Yp3=kx;Zs2 =kx;
beta = zeros(nb_modes,length(lambda));
bx = zeros(nb_modes,length(lambda));by = zeros(nb_modes,length(lambda));
e_eff = bx;beta1 = bx;
norm1xx(nb_modes,:) = zeros(1,length(lambda));norm1xy(nb_modes,:) = zeros(1,length(lambda));
norm1yx(nb_modes,:) = zeros(1,length(lambda));norm1yy(nb_modes,:) = zeros(1,length(lambda));
Sn12 = cell(1,nb_modes);Sn12_t = Sn12;Sn23 = Sn12;Sn23_t = Sn12;
rho23=Sn12;rho21=Sn12;tau21=Sn12;tau23=Sn12;
vec=zeros(length(lambda),nb_modes);t=zeros(nb_modes,length(lambda));
cf=zeros(1,length(lambda));Mat=zeros(nb_modes,nb_modes,length(lambda));
Y_alpha = Sn12;G00 = zeros(nb_modes,nb_modes,length(lambda));Gp00=G00;Mat1=G00;Mat2=G00;
Matrho21=Sn12;Matrho23=Sn12;B_rho21=Sn12;B_rho23=Sn12;
I0 = zeros(nb_modes,length(lambda));fac2x = zeros(1,nb_modes);fac1y = fac2x;
tau12=zeros(nb_modes,length(lambda));rho12=zeros(nb_difftot2,length(lambda));
mx=zeros(nb_modes,1);my=mx;facx=zeros(nb_modes);facy=facx;fbx=zeros(nb_modes,length(lambda));fby=fbx;
for ll = 1:nb_modes
    Sn12{ll} = zeros(nb_difftot2,length(lambda));Sn12_t{ll} = Sn12{ll};Sn23{ll}=Sn12{ll};Sn23_t{ll}=Sn12{ll};
    rho23{ll}=zeros(nb_modes,length(lambda));rho21{ll}=rho23{ll};
    Matrho23{ll}=zeros(nb_modes,nb_modes,length(lambda));
    tau21{ll}=Sn12{ll};tau23{ll}=Sn12{ll};
end

%% Overlap integral calculation
for pf = 1:nb_modes_TE
    inc2 = inc2+1;
    inc = 0;
    mx(inc2,:)=mx_te(inc2);
    my(inc2,:)=my_te(inc2);
    if isequal(em,'PEC')
        beta(inc2,:) = sqrt(kw2.^2-(mx(inc2,:).*pi./a).^2-(my(inc2,:).*pi./a).^2);
        bx(inc2,:) = mx(inc2,:).*pi./a+1e-10;
        by(inc2,:) = my(inc2,:).*pi./a;
    else
        if my(inc2,:)==1 && mx(inc2,:)==0
            %Approximate equation for the waveguide mode of a parralel
            %plate waveguide for the funsamental mode. See Eq. 15 in the Article
            e_eff(inc2,:) = (sqrt(e2).*sqrt(1+lambda.*1E-6./(pi.*a.*sqrt(-em)).*sqrt(1+e2./(-em)))).^2;
            aprime = a.*(1+lambda.*1E-6./(pi.*a.*sqrt((e_eff(inc2,:)-em)+(lambda.*1E-6./(2.*a)).^2)));
            beta(inc2,:) = ((kw.*sqrt(e_eff(inc2,:)-(lambda.*1E-6./(2.*aprime)).^2)));
        else
            %Nelder-Mead algorithm for higher modes
            ini_guess = sqrt(kw(1).^2.*e2-((mx(inc2,:)*pi./a(1)).^2)/1.25);
            x = [real(ini_guess) imag(ini_guess)];
            for l = 1:length(lambda)
                x0 = [x(1) x(2)];
                fun = @(x)abs(tanh(sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*e2).*a/2-mx(inc2,:).*1i.*pi/2)+e2.*sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*em(l))./(em(l).*sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*e2)));
                %options = optimset('MaxIter',1000,'TolFun',1e-5,'TolX',1e-5);
                [x] = fminsearch(fun,x0);
                beta1(inc2,l) =  x(1)+1i.*x(2);
                e_eff(inc2,l) = (beta1(inc2,l)./kw(l)).^2;
            end
            ini_guess = sqrt(e_eff(inc2,1).*kw(1).^2-((my(inc2,:)*pi./a(1)).^2)./1.25);
            x2 = [real(ini_guess) imag(ini_guess)];
            for l = 1:length(lambda)
                x02 = [x2(1) x2(2)];
                fun = @(x)abs(coth(sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*e_eff(inc2,l)).*a/2-(my(inc2,:)).*1i.*pi/2)+sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*em(l))./(sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*e_eff(inc2,l))));
                %options = optimset('MaxIter',1000,'MaxFunEvals',2000,'TolFun',1e-4,'TolX',1e-4);
                [x2] = fminsearch(fun,x02);
                beta(inc2,l) =  x2(1)+1i.*x2(2);
            end
        end
        bx(inc2,:) = sqrt(kw2.^2-kw.^2.*e_eff(inc2,:));
        by(inc2,:) = sqrt(kw.^2.*e_eff(inc2,:)-beta(inc2,:).^2);
        %alphax = sqrt(-kw.^2.*em+kw.^2.*e_eff(inc2,:));
        %alphay = sqrt(-kw.^2.*em+beta(inc2,:).^2);
    end
    %Normalization factors
    Y_alpha{inc2} = (beta(inc2,:)./(kw)).^1;
    norm1xx(inc2,:) = ((a.*bx(inc2,:)+sin(a.*bx(inc2,:)))./(2.*bx(inc2,:))).^(-1/2);
    norm1xy(inc2,:) = ((a.*by(inc2,:)+sin(a.*by(inc2,:)))./(2.*by(inc2,:))).^(-1/2);
    if mx(inc2) ==0
        norm1yx(inc2,:) = 0;
    else
        norm1yx(inc2,:) = ((a.*bx(inc2,:)-sin(a.*bx(inc2,:)))./(2.*bx(inc2,:))).^(-1/2);
    end
    norm1yy(inc2,:) = ((a.*by(inc2,:)-sin(a.*by(inc2,:)))./(2.*by(inc2,:))).^(-1/2);
    norm2 = 1./(d);
    
    if mod(my(inc2),4)==0||mod(my(inc2),4)==1
        fac1x(inc2) = 1;
    else
        fac1x(inc2) =-1;
    end
    if mod(mx(inc2),4)== 1||mod(mx(inc2),4)==2
        fac2x(inc2) = -1;
    else
        fac2x(inc2) =1;
    end
    if mod(mx(inc2),4)==0||mod(mx(inc2),4)==1
        fac2y(inc2) = 1;
    else
        fac2y(inc2) =-1;
    end
    if mod(my(inc2),4)==1||mod(my(inc2),4)==2
        fac1y(inc2) = -1;
    else
        fac1y(inc2) =1;
    end
    
    facx(inc2) = fac1x(inc2).*fac2x(inc2);
    facy(inc2) = fac1y(inc2).*fac2y(inc2);
    
    fbx(inc2,:) = by(inc2,:)./sqrt(bx(inc2,:).^2+by(inc2,:).^2);
    fby(inc2,:) = bx(inc2,:)./sqrt(bx(inc2,:).^2+by(inc2,:).^2);
    
    for  l = -nb_diffx:nb_diffx
        for ll = -nb_diffy:nb_diffy
            inc = inc+1;
            kx(inc,:) = kx0+l*kr;
            kx3(inc,:) = kx03+l*kr;
            ky(inc,:) = ky0+ll*kr;
            ky3(inc,:) = ky03+ll*kr;
            kll(inc,:) = sqrt(kx(inc,:).^2+ky(inc,:).^2);
            kll3(inc,:) = sqrt(kx3(inc,:).^2+ky3(inc,:).^2);
            kz(inc,:) = sqrt(kw1.^2-kll(inc,:).^2);
            kz3(inc,:) = sqrt(kw3.^2-kll3(inc,:).^2);
            if kx(inc,1) == 0 && ky(inc,1) == 0
                fx1(inc,1:length(lambda)) = 1;
                fx3(inc,1:length(lambda)) = 1;
                fx1(inc+nb_difftot,1:length(lambda)) = 0;
                fx3(inc+nb_difftot,1:length(lambda)) = 0;
                fy1(inc,1:length(lambda)) = 0;
                fy3(inc,1:length(lambda)) = 0;
                fy1(inc+nb_difftot,1:length(lambda)) = 1;
                fy3(inc+nb_difftot,1:length(lambda)) = 1;
            else
                fx1(inc,1:length(lambda)) = (kx(inc,:))./sqrt(kx(inc,:).^2+ky(inc,:).^2);
                fx3(inc,1:length(lambda)) = (kx3(inc,:))./sqrt(kx3(inc,:).^2+ky3(inc,:).^2);
                fx1(inc+nb_difftot,1:length(lambda)) = (-ky(inc,:))./sqrt(kx(inc,:).^2+ky(inc,:).^2);
                fx3(inc+nb_difftot,1:length(lambda)) = (-ky3(inc,:))./sqrt(kx3(inc,:).^2+ky3(inc,:).^2);
                fy1(inc,1:length(lambda)) = (ky(inc,:))./sqrt(kx(inc,:).^2+ky(inc,:).^2);
                fy3(inc,1:length(lambda)) = (ky3(inc,:))./sqrt(kx3(inc,:).^2+ky3(inc,:).^2);
                fy1(inc+nb_difftot,1:length(lambda)) = (kx(inc,:))./sqrt(kx(inc,:).^2+ky(inc,:).^2);
                fy3(inc+nb_difftot,1:length(lambda)) = (kx3(inc,:))./sqrt(kx3(inc,:).^2+ky3(inc,:).^2);
            end
            
            %Normalized Overlap Integral. Eq 19 in the article
            
            intsiny = 2.*1i.*(ky(inc,:).*sin(a.*by(inc2,:)./2).*cos(a.*ky(inc,:)./2)-by(inc2,:).*cos(a.*by(inc2,:)./2).*sin(a.*ky(inc,:)./2))./(by(inc2,:).^2-ky(inc,:).^2);
            intsiny3 = 2.*1i.*(ky3(inc,:).*sin(a.*by(inc2,:)./2).*cos(a.*ky3(inc,:)./2)-by(inc2,:).*cos(a.*by(inc2,:)./2).*sin(a.*ky3(inc,:)./2))./(by(inc2,:).^2-ky3(inc,:).^2);
            intsinx = 2.*1i.*(kx(inc,:).*sin(a.*bx(inc2,:)./2).*cos(a.*kx(inc,:)./2)-bx(inc2,:).*cos(a.*bx(inc2,:)./2).*sin(a.*kx(inc,:)./2))./(bx(inc2,:).^2-kx(inc,:).^2);
            intsinx3 = 2.*1i.*(kx3(inc,:).*sin(a.*bx(inc2,:)./2).*cos(a.*kx3(inc,:)./2)-bx(inc2,:).*cos(a.*bx(inc2,:)./2).*sin(a.*kx3(inc,:)./2))./(bx(inc2,:).^2-kx3(inc,:).^2);
            intcosx = (2.*bx(inc2,:).*sin(a.*bx(inc2,:)./2).*cos(a.*kx(inc,:)./2)-2.*kx(inc,:).*cos(a.*bx(inc2,:)./2).*sin(a.*kx(inc,:)./2))./(bx(inc2,:).^2-kx(inc,:).^2);
            intcosx3 = (2.*bx(inc2,:).*sin(a.*bx(inc2,:)./2).*cos(a.*kx3(inc,:)./2)-2.*kx3(inc,:).*cos(a.*bx(inc2,:)./2).*sin(a.*kx3(inc,:)./2))./(bx(inc2,:).^2-kx3(inc,:).^2);
            intcosy = (2.*by(inc2,:).*sin(a.*by(inc2,:)./2).*cos(a.*ky(inc,:)./2)-2.*ky(inc,:).*cos(a.*by(inc2,:)./2).*sin(a.*ky(inc,:)./2))./(by(inc2,:).^2-ky(inc,:).^2);
            intcosy3 = (2.*by(inc2,:).*sin(a.*by(inc2,:)./2).*cos(a.*ky3(inc,:)./2)-2.*ky3(inc,:).*cos(a.*by(inc2,:)./2).*sin(a.*ky3(inc,:)./2))./(by(inc2,:).^2-ky3(inc,:).^2);
            
            if mod(mx(inc2,:),2) == 0 && mod(my(inc2,:),2)==1
                Sn12{inc2}(inc,1:length(lambda)) = facx(inc2).*fx1(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx.*intcosy - facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx.*intsiny+1e-10;
                Sn23{inc2}(inc,1:length(lambda)) = facx(inc2).*fx3(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx3.*intcosy3 - facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx3.*intsiny3+1e-10;
                Sn12{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx1(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx.*intcosy - facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx.*intsiny+1e-10;
                Sn23{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx3(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx3.*intcosy3 - facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx3.*intsiny3+1e-10;
            elseif mod(mx(inc2,:),2) == 1 && mod(my(inc2,:),2)==1
                Sn12{inc2}(inc,1:length(lambda)) = facx(inc2).*fx1(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx.*intcosy - facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx.*intsiny+1e-10;
                Sn23{inc2}(inc,1:length(lambda)) = facx(inc2).*fx3(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx3.*intcosy3 - facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx3.*intsiny3+1e-10;
                Sn12{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx1(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx.*intcosy - facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx.*intsiny+1e-10;
                Sn23{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx3(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx3.*intcosy3 - facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx3.*intsiny3+1e-10;
            elseif mod(mx(inc2,:),2) == 1 && mod(my(inc2,:),2)==0
                Sn12{inc2}(inc,1:length(lambda)) = facx(inc2).*fx1(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx.*intsiny - facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx.*intcosy+1e-10;
                Sn23{inc2}(inc,1:length(lambda)) = facx(inc2).*fx3(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx3.*intsiny3 - facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx3.*intcosy3+1e-10;
                Sn12{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx1(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx.*intsiny - facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx.*intcosy+1e-10;
                Sn23{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx3(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx3.*intsiny3 - facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx3.*intcosy3+1e-10;
            else
                Sn12{inc2}(inc,1:length(lambda)) = facx(inc2).*fx1(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx.*intsiny - facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx.*intcosy+1e-10;
                Sn23{inc2}(inc,1:length(lambda)) = facx(inc2).*fx3(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx3.*intsiny3 - facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx3.*intcosy3+1e-10;
                Sn12{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx1(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx.*intsiny - facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx.*intcosy+1e-10;
                Sn23{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx3(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx3.*intsiny3 - facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx3.*intcosy3+1e-10;
            end
            Sn12_t{inc2}(inc,1:length(lambda)) = conj(Sn12{inc2}(inc,1:length(lambda)));
            Sn23_t{inc2}(inc,1:length(lambda)) = conj(Sn23{inc2}(inc,1:length(lambda)));
            Sn12_t{inc2}(inc+nb_difftot,1:length(lambda)) = conj(Sn12{inc2}(inc+nb_difftot,1:length(lambda)));
            Sn23_t{inc2}(inc+nb_difftot,1:length(lambda)) = conj(Sn23{inc2}(inc+nb_difftot,1:length(lambda)));
            
            %Plane Wave Admittances.
            
            Yp(inc,:) = (kw.*e1./kz(inc,:)).^1;
            Yp(inc+nb_difftot,:) = (kw./kz(inc,:)).^(-1);
            Zs2(inc,:) = Zs;
            Yp3(inc,:) = (kw.*e3./kz3(inc,:)).^1;
            Yp3(inc+nb_difftot,:) = (kw./kz3(inc,:)).^(-1);
            Zs2(inc+nb_difftot,:) = Zs;
        end
    end
end

%MODE TM
inc1 =0;
for pf = 1:nb_modes_TM
    inc2 = inc2+1;
    inc = 0;
    inc1 = inc1+1;
    mx(inc2,:) = mx_tm(inc1);
    my(inc2,:) = my_tm(inc1);
    
    if isequal(em,'PEC')
        beta(inc2,:) = sqrt(kw2.^2-(mx(inc2,:).*pi./a).^2-(my(inc2,:).*pi./a).^2);
        bx(inc2,:) = mx(inc2,:).*pi./a;
        by(inc2,:) = my(inc2,:).*pi./a;
    else
        %Nelder-Mead algorithm for higher modes
        ini_guess = sqrt(kw(1).^2.*e2-((mx(inc2,:)*pi./a(1)).^2)/1.25);
        x = [real(ini_guess) imag(ini_guess)];
        %x = [kw2 0];
        for l = 1:length(lambda)
            x0 = [x(1) x(2)];
            fun = @(x)abs(tanh(sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*e2).*a/2-mx(inc2,:).*1i.*pi/2)+e2.*sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*em(l))./(em(l).*sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*e2)));
            %options = optimset('MaxIter',1000,'TolFun',1e-5,'TolX',1e-5);
            [x] = fminsearch(fun,x0);
            beta1(inc2,l) =  x(1)+1i.*x(2);
            e_eff(inc2,l) = (beta1(inc2,l)./kw(l)).^2;
        end
        ini_guess = sqrt(e_eff(inc2,1).*kw(1).^2-((my(inc2,:)*pi./a(1)).^2)./1.25);
        x2 = [real(ini_guess) imag(ini_guess)];
        for l = 1:length(lambda)
            x02 = [x2(1) x2(2)];
            fun = @(x)abs(coth(sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*e_eff(inc2,l)).*a/2-(my(inc2,:)).*1i.*pi/2)+sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*em(l))./(sqrt((x(1)+1i.*x(2)).^2-kw(l).^2.*e_eff(inc2,l))));
            %options = optimset('MaxIter',1000,'MaxFunEvals',2000,'TolFun',1e-5,'TolX',1e-5);
            [x2] = fminsearch(fun,x02);
            beta(inc2,l) =  (x2(1)+1i.*x2(2));
        end
        bx(inc2,:) = sqrt(kw2.^2-kw.^2.*e_eff(inc2,:));
        by(inc2,:) = sqrt(kw.^2.*e_eff(inc2,:)-beta(inc2,:).^2);
        alphax = sqrt(-kw.^2.*em+kw.^2.*e_eff(inc2,:));
        alphay = sqrt(-kw.^2.*em+beta(inc2,:).^2);
    end
    
    %Normalization factors

    Y_alpha{inc2} = ((beta(inc2,:)./(kw.*e2)).^(-1));
    norm1xx(inc2,:) = ((a.*bx(inc2,:)+sin(a.*bx(inc2,:)))./(2.*bx(inc2,:))).^(-1/2);
    norm1xy(inc2,:) = ((a.*by(inc2,:)+sin(a.*by(inc2,:)))./(2.*by(inc2,:))).^(-1/2);
    norm1yx(inc2,:) = ((a.*bx(inc2,:)-sin(a.*bx(inc2,:)))./(2.*bx(inc2,:))).^(-1/2);
    norm1yy(inc2,:) = ((a.*by(inc2,:)-sin(a.*by(inc2,:)))./(2.*by(inc2,:))).^(-1/2);
    
    if mod(my(inc2),4)==0||mod(my(inc2),4)==1
        fac1x(inc2) = 1;
    else
        fac1x(inc2) =-1;
    end
    if mod(mx(inc2),4)== 1||mod(mx(inc2),4)==2
        fac2x(inc2)=-1;
    else
        fac2x(inc2) =1;
    end
    if mod(mx(inc2),4)==0||mod(mx(inc2),4)==1
        fac2y(inc2) = 1;
    else
        fac2y(inc2) =-1;
    end
    if mod(my(inc2),4)==1||mod(my(inc2),4)==2
        fac1y(inc2) = -1;
    else
        fac1y(inc2) =1;
    end
    
    facx(inc2) = fac1x(inc2).*fac2x(inc2);
    facy(inc2) = fac1y(inc2).*fac2y(inc2);
    
    fbx(inc2,:) = bx(inc2,:)./sqrt(bx(inc2,:).^2+by(inc2,:).^2);
    fby(inc2,:) = by(inc2,:)./sqrt(bx(inc2,:).^2+by(inc2,:).^2);
    
    norm2 = 1./(d);
    for  l = -nb_diffx:nb_diffx
        for ll = -nb_diffy:nb_diffy
            inc = inc+1;
            kx(inc,:) = kx0+l*kr;
            kx3(inc,:) = kx03+l*kr;
            ky(inc,:) = ky0+ll*kr;
            ky3(inc,:) = ky03+ll*kr;
            kll(inc,:) = sqrt(kx(inc,:).^2+ky(inc,:).^2);
            kll3(inc,:) = sqrt(kx3(inc,:).^2+ky3(inc,:).^2);
            kz(inc,:) = sqrt(kw1.^2-kll(inc,:).^2);
            kz3(inc,:) = sqrt(kw3.^2-kll3(inc,:).^2);
            if kx(inc,1) == 0 && ky(inc,1) == 0
                fx1(inc,1:length(lambda)) = 1;
                fx3(inc,1:length(lambda)) = 1;
                fx1(inc+nb_difftot,1:length(lambda)) = 0;
                fx3(inc+nb_difftot,1:length(lambda)) = 0;
                fy1(inc,1:length(lambda)) = 0;
                fy3(inc,1:length(lambda)) = 0;
                fy1(inc+nb_difftot,1:length(lambda)) = 1;
                fy3(inc+nb_difftot,1:length(lambda)) = 1;
            else
                fx1(inc,1:length(lambda)) = (kx(inc,:))./sqrt(kx(inc,:).^2+ky(inc,:).^2);
                fx3(inc,1:length(lambda)) = (kx3(inc,:))./sqrt(kx3(inc,:).^2+ky3(inc,:).^2);
                fx1(inc+nb_difftot,1:length(lambda)) = (-ky(inc,:))./sqrt(kx(inc,:).^2+ky(inc,:).^2);
                fx3(inc+nb_difftot,1:length(lambda)) = (-ky3(inc,:))./sqrt(kx3(inc,:).^2+ky3(inc,:).^2);
                fy1(inc,1:length(lambda)) = (ky(inc,:))./sqrt(kx(inc,:).^2+ky(inc,:).^2);
                fy3(inc,1:length(lambda)) = (ky3(inc,:))./sqrt(kx3(inc,:).^2+ky3(inc,:).^2);
                fy1(inc+nb_difftot,1:length(lambda)) = (kx(inc,:))./sqrt(kx(inc,:).^2+ky(inc,:).^2);
                fy3(inc+nb_difftot,1:length(lambda)) = (kx3(inc,:))./sqrt(kx3(inc,:).^2+ky3(inc,:).^2);
            end
            
            %Normalized Overlap Integral. Eq 19 in the arctile
            
            intsiny = 2.*1i.*(ky(inc,:).*sin(a.*by(inc2,:)./2).*cos(a.*ky(inc,:)./2)-by(inc2,:).*cos(a.*by(inc2,:)./2).*sin(a.*ky(inc,:)./2))./(by(inc2,:).^2-ky(inc,:).^2);
            intsiny3 = 2.*1i.*(ky3(inc,:).*sin(a.*by(inc2,:)./2).*cos(a.*ky3(inc,:)./2)-by(inc2,:).*cos(a.*by(inc2,:)./2).*sin(a.*ky3(inc,:)./2))./(by(inc2,:).^2-ky3(inc,:).^2);
            intsinx = 2.*1i.*(kx(inc,:).*sin(a.*bx(inc2,:)./2).*cos(a.*kx(inc,:)./2)-bx(inc2,:).*cos(a.*bx(inc2,:)./2).*sin(a.*kx(inc,:)./2))./(bx(inc2,:).^2-kx(inc,:).^2);
            intsinx3 = 2.*1i.*(kx3(inc,:).*sin(a.*bx(inc2,:)./2).*cos(a.*kx3(inc,:)./2)-bx(inc2,:).*cos(a.*bx(inc2,:)./2).*sin(a.*kx3(inc,:)./2))./(bx(inc2,:).^2-kx3(inc,:).^2);
            intcosx = (2.*bx(inc2,:).*sin(a.*bx(inc2,:)./2).*cos(a.*kx(inc,:)./2)-2.*kx(inc,:).*cos(a.*bx(inc2,:)./2).*sin(a.*kx(inc,:)./2))./(bx(inc2,:).^2-kx(inc,:).^2);
            intcosx3 = (2.*bx(inc2,:).*sin(a.*bx(inc2,:)./2).*cos(a.*kx3(inc,:)./2)-2.*kx3(inc,:).*cos(a.*bx(inc2,:)./2).*sin(a.*kx3(inc,:)./2))./(bx(inc2,:).^2-kx3(inc,:).^2);
            intcosy = (2.*by(inc2,:).*sin(a.*by(inc2,:)./2).*cos(a.*ky(inc,:)./2)-2.*ky(inc,:).*cos(a.*by(inc2,:)./2).*sin(a.*ky(inc,:)./2))./(by(inc2,:).^2-ky(inc,:).^2);
            intcosy3 = (2.*by(inc2,:).*sin(a.*by(inc2,:)./2).*cos(a.*ky3(inc,:)./2)-2.*ky3(inc,:).*cos(a.*by(inc2,:)./2).*sin(a.*ky3(inc,:)./2))./(by(inc2,:).^2-ky3(inc,:).^2);
            
            if mod(mx(inc2,:),2) == 0 && mod(my(inc2,:),2)==1
                
                Sn12{inc2}(inc,1:length(lambda)) = facx(inc2).*fx1(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx.*intcosy + facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx.*intsiny+1e-10;
                Sn23{inc2}(inc,1:length(lambda)) = facx(inc2).*fx3(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx3.*intcosy3 + facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx3.*intsiny3+1e-10;
                Sn12{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx1(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx.*intcosy + facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx.*intsiny+1e-10;
                Sn23{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx3(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx3.*intcosy3 + facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx3.*intsiny3+1e-10;
                
            elseif mod(mx(inc2,:),2) == 1 && mod(my(inc2,:),2)==1
                
                Sn12{inc2}(inc,1:length(lambda)) = facx(inc2).*fx1(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx.*intcosy + facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx.*intsiny+1e-10;
                Sn23{inc2}(inc,1:length(lambda)) = facx(inc2).*fx3(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx3.*intcosy3 + facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx3.*intsiny3+1e-10;
                Sn12{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx1(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx.*intcosy + facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx.*intsiny+1e-10;
                Sn23{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx3(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx3.*intcosy3 + facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx3.*intsiny3+1e-10;
                
            elseif mod(mx(inc2,:),2) == 1 && mod(my(inc2,:),2)==0
                
                Sn12{inc2}(inc,1:length(lambda)) = facx(inc2).*fx1(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx.*intsiny + facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx.*intcosy+1e-10;
                Sn23{inc2}(inc,1:length(lambda)) = facx(inc2).*fx3(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx3.*intsiny3 + facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx3.*intcosy3+1e-10;
                Sn12{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx1(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx.*intsiny + facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx.*intcosy+1e-10;
                Sn23{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx3(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intsinx3.*intsiny3 + facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intcosx3.*intcosy3+1e-10;
                
            else
                Sn12{inc2}(inc,1:length(lambda)) = facx(inc2).*fx1(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx.*intsiny + facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx.*intcosy+1e-10;
                Sn23{inc2}(inc,1:length(lambda)) = facx(inc2).*fx3(inc,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx3.*intsiny3 + facy(inc2).*fy1(inc,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx3.*intcosy3+1e-10;
                Sn12{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx1(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx.*intsiny + facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx.*intcosy+1e-10;
                Sn23{inc2}(inc+nb_difftot,1:length(lambda)) = facx(inc2).*fx3(inc+nb_difftot,:).*fbx(inc2,:).*norm1xx(inc2,:).*norm1xy(inc2,:).*norm2.*intcosx3.*intsiny3 + facy(inc2).*fy1(inc+nb_difftot,:).*fby(inc2,:).*norm1yx(inc2,:).*norm1yy(inc2,:).*norm2.*intsinx3.*intcosy3+1e-10;
            end
            
            Sn12_t{inc2}(inc,1:length(lambda)) = conj(Sn12{inc2}(inc,1:length(lambda)));
            Sn23_t{inc2}(inc,1:length(lambda)) = conj(Sn23{inc2}(inc,1:length(lambda)));
            Sn12_t{inc2}(inc+nb_difftot,1:length(lambda)) = conj(Sn12{inc2}(inc+nb_difftot,1:length(lambda)));
            Sn23_t{inc2}(inc+nb_difftot,1:length(lambda)) = conj(Sn23{inc2}(inc+nb_difftot,1:length(lambda)));
            
            %Plane Wave Admittances.
            
            Yp(inc,:) = (kw.*e1./kz(inc,:)).^1;
            Yp(inc+nb_difftot,:) = (kw./kz(inc,:)).^(-1);
            Zs2(inc,:) = Zs;
            Yp3(inc,:) = (kw.*e3./kz3(inc,:)).^1;
            Yp3(inc+nb_difftot,:) = (kw./kz3(inc,:)).^(-1);
            Zs2(inc+nb_difftot,:) = Zs;
        end
    end
end

%% Scattering coefficient calculation
%Coupling Factor G. Eq 25 in the article

for l = 1:nb_modes
    for ll = 1:nb_modes
        G00(l,ll,:) = 1i.*sum(Yp.*(Sn12{l}.*Sn12_t{ll})./(1+Yp.*Zs2));
        Gp00(l,ll,:) = 1i.*sum(Yp3.*(Sn23{l}.*Sn23_t{ll})./(1+Yp3.*Zs2));
        Mat1(l,ll,:) = squeeze(G00(l,ll,:)).'.*(1-Zs.*Y_alpha{l});
        Mat2(l,ll,:) = squeeze(Gp00(l,ll,:)).'.*(1-Zs.*Y_alpha{l});
    end
    Mat1(l,l,:) = squeeze(Mat1(l,l,:)) + 1i.*Y_alpha{l}.';
    % Use of sqrt(Yp)in order to normalize the energy flux
    I0(l,:) = 2.*1i.*Sn12{l}(n0S,:).*sqrt(Yp(n0S,:))./(1+Yp(n0S,:).*Zs);
end
%Matrix definition used for the calculation of reflection coefficient 2-3
%and 2-1. See Eq 23 
for l = 1:nb_modes
    for ll = 1:nb_modes
        for lll = 1:nb_modes
            Matrho23{l}(ll,lll,:) = (1-Zs.*Y_alpha{lll}).*squeeze(Gp00(ll,lll,:)).';
            Matrho21{l}(ll,lll,:) = (1-Zs.*Y_alpha{lll}).*squeeze(G00(ll,lll,:)).';
            if ll == l
                B_rho23{l}(ll,:) = 1i.*Y_alpha{ll}-(1+Zs.*Y_alpha{l}).*squeeze(Gp00(ll,l,:)).';
                B_rho21{l}(ll,:) = 1i.*Y_alpha{ll}-(1+Zs.*Y_alpha{l}).*squeeze(G00(ll,l,:)).';
            else
                B_rho23{l}(ll,:) = -(1+Zs.*Y_alpha{l}).*squeeze(Gp00(ll,l,:)).';
                B_rho21{l}(ll,:) = -(1+Zs.*Y_alpha{l}).*squeeze(G00(ll,l,:)).';
            end
        end
        Matrho23{l}(ll,ll,:) = squeeze(Matrho23{l}(ll,ll,:)).'+1i.*Y_alpha{ll};
        Matrho21{l}(ll,ll,:) = squeeze(Matrho21{l}(ll,ll,:)).'+1i.*Y_alpha{ll};
    end
end
%Calculation of the scattering coefficient t12 and p23, p21
%See Eq 21,23
for l = 1:length(lambda)
    tau12(:,l) = Mat1(:,:,l)\I0(:,l);
    for ll = 1:nb_modes
        rho23{ll}(:,l) = Matrho23{ll}(:,:,l)\B_rho23{ll}(:,l);
        rho21{ll}(:,l) = Matrho21{ll}(:,:,l)\B_rho21{ll}(:,l);
    end
end
inc12 =0;
%Calculation of the scattering coefficient p12
%See Eq 22
for ll = 1:nb_difftot2
    inc12 = inc12+1;
    somme12 = 0;
    for lll = 1:nb_modes
        somme12 =  somme12+Sn12_t{lll}(ll,:).*tau12(lll,:).*(1-Zs.*Y_alpha{lll});
    end
    if ll == n0S
        rho12(inc12,:) = (-(1-Zs.*Yp(n0S,:))+somme12)./(1+Zs.*Yp(ll,:));
    else
        rho12(inc12,:) =somme12./(1+Zs.*Yp(ll,:))  ;
    end
end
%Calculation of the scattering coefficient t23,t21
%See Eq 24
for l = 1:nb_modes
    for ll = 1:nb_difftot2
        somme =0;
        somme2 = 0;
        for lll = 1:nb_modes
            somme =somme +rho23{l}(lll,:).*Sn23_t{lll}(ll,:).*(1-Zs.*Y_alpha{lll});
            somme2 =somme2 +rho21{l}(lll,:).*Sn12_t{lll}(ll,:).*(1-Zs.*Y_alpha{lll});
        end
        tau23{l}(ll,:) = (Sn23_t{l}(ll,:).*(1+Zs.*Y_alpha{l})+somme)./(1+Zs.*Yp3(ll,:));
        tau21{l}(ll,:) = (Sn12_t{l}(ll,:).*(1+Zs.*Y_alpha{l})+somme2)./(1+Zs.*Yp(ll,:));
    end
    %Normalization of the energy flow, output
    tau23{l}(n0S,:) = tau23{l}(n0S,:).*sqrt(Yp3(n0S,:));
end

B = zeros(nb_modes,length(lambda));
A = zeros(nb_modes,1);
%Calculation of the transmission factor t
%See Eqs. 28-31
for l = 1:nb_modes
    inc1 = 0;
    for ll = 1:nb_modes
        for lll = 1:nb_modes
            inc2 = 0;
            inc1 = inc1+1;
            vec(:,inc1) = rho23{l}(lll,:).*exp(1i.*beta(lll,:).*h).*rho21{lll}(ll,:).*exp(1i.*beta(ll,:).*h);
            for vv = 1:nb_difftot2
                t(inc1,:,vv) = tau23{ll}(vv,:)./tau23{l}(vv,:);
            end
            for iv = 1:nb_modes
                for v = 1:nb_modes
                    inc2 = inc2+1;
                    Mat(inc1,inc2,:) = rho23{ll}(v,:).*exp(1i.*beta(v,:).*h).*rho21{v}(iv,:).*exp(1i.*beta(iv,:).*h);
                end
            end
        end
    end
    
    for lam = 1:length(lambda)
        for vv = 1:nb_difftot2
            cf(lam,vv) = vec(lam,:)/(eye(nb_modes.^2)-Mat(:,:,lam))*t(:,lam,vv);
        end
    end
    for vv = 1:nb_difftot2
        trans(vv,:) = trans(vv,:)+tau12(l,:).*exp(1i.*beta(l,:).*h).*tau23{l}(vv,:).*(1+cf(:,vv).');
    end
end

%Calculation of the reflection coefficient and modal amplitude B.
%See eq 32,34
clear t cf
t=zeros(nb_modes,length(lambda));cf=zeros(1,length(lambda));

for l = 1:nb_modes
    for ll = 1:nb_modes
        inc1 = 0;
        for lll = 1:nb_modes
            for iv = 1:nb_modes
                inc1 = inc1+1;
                vec(:,inc1) = rho23{l}(lll,:).*exp(1i.*beta(lll,:).*h).*rho21{lll}(iv,:).*exp(1i.*beta(iv,:).*h);
                inc2 = 0;
                t(inc1,1:length(lambda)) = rho23{iv}(ll,:)./rho23{l}(ll,:);
                for v = 1:nb_modes
                    for vi = 1:nb_modes
                        inc2 = inc2+1;
                        Mat(inc1,inc2,:) = rho23{iv}(v,:).*exp(1i.*beta(v,:).*h).*rho21{v}(vi,:).*exp(1i.*beta(vi,:).*h);
                    end
                end
            end
        end
        for lam = 1:length(lambda)
            cf(lam) = vec(lam,:)/(eye(nb_modes.^2)-Mat(:,:,lam))*t(:,lam);
        end
        for vv = 1:nb_difftot2
            refl(vv,:) = refl(vv,:)+tau12(l,:).*exp(1i.*beta(l,:).*h).*rho23{l}(ll,:).*exp(1i.*beta(ll,:).*h).*tau21{ll}(vv,:).*(1+cf);
        end
        B(ll,:) = B(ll,:)+ tau12(l,:).*exp(1i.*h.*beta(l,:)).*rho23{l}(ll,:).*exp(1i.*h.*beta(ll,:)./2).*(1+cf);
    end
end

refl = refl+rho12;

if cx > 0
[x_p,y_p] = findpeaks(abs(trans(n0S,:)).^2);
y_p = y_p(end+1-cx);
end

vec2=complex(1,nb_modes);cf2=complex(1);
Mat2=zeros(nb_modes,nb_modes,1);t2=complex(nb_modes,1);

%Calculation of Modal Amplitude A
%See Eq. 34

for l = 1:nb_modes
    somme_A =0;
    for ll = 1:nb_modes
        for lll = 1:nb_modes
            inc2 = 0;
            for iv = 1:nb_modes
                for v = 1:nb_modes
                    inc2 = inc2+1;
                    inc3 = 0;
                    vec2(1,inc2) = rho23{ll}(iv,y_p).*exp(1i.*beta(iv,y_p).*h).*rho21{iv}(v,y_p).*exp(1i.*beta(v,y_p).*h);
                    for vi = 1:nb_modes
                        for vii = 1:nb_modes
                            inc3 = inc3+1;
                            Mat2(inc2,inc3,1) = rho23{v}(vi,y_p).*exp(1i.*beta(vi,y_p).*h).*rho21{vi}(vii,y_p).*exp(1i.*beta(vii,y_p).*h);
                            t2(inc3,1) = rho23{vii}(lll,y_p)./rho23{ll}(lll,y_p);
                        end
                    end
                end
            end
            for lam = 1:length(y_p)
                cf2(lam) =vec2(1,:)/(eye(nb_modes.^2)-Mat2(:,:,1))*t2(:,1);
            end
            somme_A = somme_A + tau12(ll,y_p)./tau12(l,y_p).*exp(1i.*beta(ll,y_p).*h).*rho23{ll}(lll,y_p).*exp(1i.*beta(lll,y_p).*h).*rho21{lll}(l,y_p).*(1+cf2);
        end
    end
    A(l) = tau12(l,y_p).*exp(1i.*beta(l,y_p).*h/2).*(1+somme_A);
end

%% Calculation - Field Expression
nb_plane = 1;
clims = [0 0];

if pp<3
    if pp ==1
        x = -d/2:step:d/2;
        y = plane;
        z = -dom:step:dom;
        inc4 = 0;
        for l = 1:length(x)
            if x(l) > -a/2 && x(l) < a/2
                start = l;
                inc4 = inc4+1;
            end
        end
        xspread = start-inc4+1:start;
        yspread = 1;
    else
        x = plane;
        y = -d/2:step:d/2;
        z = -dom:step:dom;
        xspread = 1;
        inc4 = 0;
        for l = 1:length(y)
            if y(l) > -a/2 && y(l) < a/2
                start = l;
                inc4 = inc4+1;
            end
        end
        yspread = start-inc4+1:start;
    end
    
    inc = 0;
    inc1 = 0;inc2 =0;inc3 =0;inc4 = 0;
    
    for l = 1:length(z)
        if z(l)<= -h/2
            inc1 = inc1+1;
        elseif z(l) < h/2
            inc2 = inc2+1;
        else
            inc3 = inc3+1;
        end
    end
    
    z1 = z(1:inc1);
    z2 = z(inc1+1:inc1+inc2);
    z3 = z(inc1+1+inc2:inc1+inc2+inc3);
    
    x2 = x(xspread);
    y2 = y(yspread);
    [X1,Z1,Y1] = meshgrid(x,z1,y);
    [X2,Z2,Y2] = meshgrid(x2,z2,y2);
    [X3,Z3,Y3] = meshgrid(x,z3,y);
    
    %Hy = complex(zeros(nb_difftot*2,1,length(z),length(x),length(y)));
    Ex = complex(zeros(nb_difftot2,1,length(z),length(x),length(y)));
    Ez = complex(zeros(nb_difftot2,1,length(z),length(x),length(y)));
    Ey = complex(zeros(nb_difftot2,1,length(z),length(x),length(y)));
    
    for ll = y_p
        inc_lam = 1;
        for l2 = 1:nb_difftot2
            crit_l = floor((l2-1)./nb_difftot);
            l =  mod(l2-1,nb_difftot)+1;
            if crit_l ==0
                if l == n0S
                    %Hy(l,inc_lam,1:inc1,:) = 1./sqrt(d).*((Yp(l,ll)).*exp(1i.*k(l,ll).*(X1)).*exp(1i.*kz(l,ll).*(Z1+h/2))+ refl(l,ll).*-Yp(l,ll).*exp(1i.*k(l,ll).*(X1)).*exp(-1i.*kz(l,ll).*(Z1+h/2)));
                    %Hy(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:) = 1./sqrt(d).*((Yp3(l,ll)).*trans(l,ll).*exp(1i.*k(l,ll).*(X3)).*exp(1i.*kz(l,ll).*(Z3-h/2)));
                    Ex(l,inc_lam,1:inc1,:,:) = fx1(l,ll)./d.*(exp(1i.*kx(l,ll).*(X1)).*exp(1i.*ky(l,ll).*(Y1)).*exp(1i.*kz(l,ll).*(Z1+h/2))+ refl(l,ll).*exp(1i.*kx(l,ll).*(X1)).*exp(1i.*ky(l,ll).*(Y1)).*exp(-1i.*kz(l,ll).*(Z1+h/2+step/2)));
                    Ex(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:,:) = fx3(l,ll)./d.*(1.*trans(l,ll).*exp(1i.*kx3(l,ll).*(X3)).*exp(1i.*ky3(l,ll).*(Y3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
                    Ey(l,inc_lam,1:inc1,:,:) = fy1(l,ll)./d.*(exp(1i.*kx(l,ll).*(X1)).*exp(1i.*ky(l,ll).*(Y1)).*exp(1i.*kz(l,ll).*(Z1+h/2))+ refl(l,ll).*exp(1i.*kx(l,ll).*(X1)).*exp(1i.*ky(l,ll).*(Y1)).*exp(-1i.*kz(l,ll).*(Z1+h/2+step/2)));
                    Ey(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:,:) = fy3(l,ll)./d.*(1.*trans(l,ll).*exp(1i.*kx3(l,ll).*(X3)).*exp(1i.*ky3(l,ll).*(Y3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
                    Ez(l,inc_lam,1:inc1,:,:) = -kll(l,ll)./kz(l,ll)./d.*(exp(1i.*kx(l,ll).*(X1)).*exp(1i.*ky(l,ll).*(Y1)).*exp(1i.*kz(l,ll).*(Z1+h/2))-refl(l,ll).*exp(1i.*kx(l,ll).*(X1)).*exp(1i.*ky(l,ll).*(Y1)).*exp(-1i.*kz(l,ll).*(Z1+h/2+step/2)));
                    Ez(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:,:) = kll3(l,ll)./kz3(l,ll)./d.*(1.*trans(l,ll).*exp(1i.*kx3(l,ll).*(X3)).*exp(1i.*ky3(l,ll).*(Y3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
                else
                    %Hy(l,inc_lam,1:inc1,:) = 1./sqrt(d).*(-Yp(l,ll).*refl(l,ll).*exp(1i.*k(l,ll).*(X1)).*exp(-1i.*kz(l,ll).*(Z1+h/2)));
                    %Hy(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:) = 1./sqrt(d).*(Yp3(l,ll).*trans(l,ll).*exp(1i.*k(l,ll).*(X3)).*exp(1i.*kz(l,ll).*(Z3-h/2)));
                    Ex(l,inc_lam,1:inc1,:,:) = fx1(l,ll)./d.*(refl(l,ll).*exp(1i.*kx(l,ll).*(X1)).*exp(1i.*ky(l,ll).*(Y1)).*exp(-1i.*kz(l,ll).*(Z1+h/2+step/2)));
                    Ex(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:,:) = fx3(l,ll)./d.*(trans(l,ll).*exp(1i.*kx3(l,ll).*(X3)).*exp(1i.*ky3(l,ll).*(Y3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
                    Ey(l,inc_lam,1:inc1,:,:) = fy1(l,ll)./d.*(refl(l,ll).*exp(1i.*kx(l,ll).*(X1)).*exp(1i.*ky(l,ll).*(Y1)).*exp(-1i.*kz(l,ll).*(Z1+h/2+step/2)));
                    Ey(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:,:) = fy3(l,ll)./d.*(trans(l,ll).*exp(1i.*kx3(l,ll).*(X3)).*exp(1i.*ky3(l,ll).*(Y3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
                    Ez(l,inc_lam,1:inc1,:,:) = kll(l,ll)./kz(l,ll)./d.*(refl(l,ll).*exp(1i.*kx(l,ll).*(X1)).*exp(1i.*ky(l,ll).*(Y1)).*exp(-1i.*kz(l,ll).*(Z1+h/2+step/2)));
                    Ez(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:,:) = kll3(l,ll)./kz3(l,ll)./d.*(trans(l,ll).*exp(1i.*kx3(l,ll).*(X3)).*exp(1i.*ky3(l,ll).*(Y3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
                end
            else
                %Hy(l,inc_lam,1:inc1,:) = 1./sqrt(d).*(-Yp(l,ll).*refl(l,ll).*exp(1i.*k(l,ll).*(X1)).*exp(-1i.*kz(l,ll).*(Z1+h/2)));
                %Hy(l,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:) = 1./sqrt(d).*(Yp3(l,ll).*trans(l,ll).*exp(1i.*k(l,ll).*(X3)).*exp(1i.*kz(l,ll).*(Z3-h/2)));
                Ex(l+nb_difftot,inc_lam,1:inc1,:,:) = fx1(nb_difftot+l,ll)./d.*(refl(l+nb_difftot,ll).*exp(1i.*kx(l,ll).*(X1)).*exp(1i.*ky(l,ll).*(Y1)).*exp(-1i.*kz(l,ll).*(Z1+h/2+step/2)));
                Ex(l+nb_difftot,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:,:) = fx3(nb_difftot+l,ll)./d.*(1.*trans(l+nb_difftot,ll).*exp(1i.*kx3(l,ll).*(X3)).*exp(1i.*ky3(l,ll).*(Y3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
                Ey(l+nb_difftot,inc_lam,1:inc1,:,:) = fy1(nb_difftot+l,ll)./d.*(refl(l+nb_difftot,ll).*exp(1i.*kx(l,ll).*(X1)).*exp(1i.*ky(l,ll).*(Y1)).*exp(-1i.*kz(l,ll).*(Z1+h/2+step/2)));
                Ey(l+nb_difftot,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:,:) = fy3(nb_difftot+l,ll)./d.*(1.*trans(l+nb_difftot,ll).*exp(1i.*kx3(l,ll).*(X3)).*exp(1i.*ky3(l,ll).*(Y3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
                Ez(l+nb_difftot,inc_lam,1:inc1,:,:) = 0./kz(l,ll)./d.*(refl(l+nb_difftot,ll).*exp(1i.*kx(l,ll).*(X1)).*exp(1i.*ky(l,ll).*(Y1)).*exp(-1i.*kz(l,ll).*(Z1+h/2+step/2)));
                Ez(l+nb_difftot,inc_lam,inc1+inc2+1:inc1+inc2+inc3,:,:) = 0./kz3(l,ll)./d.*(trans(l+nb_difftot,ll).*exp(1i.*kx3(l,ll).*(X3)).*exp(1i.*ky3(l,ll).*(Y3)).*exp(1i.*kz3(l,ll).*(Z3-h/2-step/2)));
            end
        end
        
        for l = 1:nb_modes
            if plane < a/2 && plane > -a/2
                if mod(mx(l),2) == 0 && mod(my(l),2)==1
                    Ex(l,inc_lam,inc1+1:inc1+inc2,xspread,yspread) = facx(l).*fbx(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1xx(l,ll).*norm1xy(l,ll).*cos(by(l,ll).*Y2).*cos(bx(l,ll).*X2));
                    Ey(l,inc_lam,inc1+1:inc1+inc2,xspread,yspread) = facy(l).*fby(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1yx(l,ll).*norm1yy(l,ll).*sin(by(l,ll).*Y2).*sin(bx(l,ll).*X2));
                    if l > nb_modes_TE
                        Ez(l,inc_lam,inc1+1:inc1+inc2,xspread,yspread) = fac1x(l).*fac2y(l).*1i.*sqrt(bx(l,ll).^2+by(l,ll).^2)./beta(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1xy(l,ll).*norm1yx(l,ll).*cos(by(l,ll).*Y2).*sin(bx(l,ll).*X2));
                    end
                elseif mod(mx(l),2) == 1 && mod(my(l),2)==1
                    Ex(l,inc_lam,inc1+1:inc1+inc2,xspread,yspread) = facx(l).*fbx(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1xx(l,ll).*norm1xy(l,ll).*cos(by(l,ll).*Y2).*sin(bx(l,ll).*X2));
                    Ey(l,inc_lam,inc1+1:inc1+inc2,xspread,yspread) = facy(l).*fby(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1yx(l,ll).*norm1yy(l,ll).*sin(by(l,ll).*Y2).*cos(bx(l,ll).*X2));
                    if l > nb_modes_TE
                        Ez(l,inc_lam,inc1+1:inc1+inc2,xspread,yspread) = fac1x(l).*fac2y(l).*1i.*sqrt(bx(l,ll).^2+by(l,ll).^2)./beta(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1xy(l,ll).*norm1yx(l,ll).*cos(by(l,ll).*Y2).*cos(bx(l,ll).*X2));
                    end
                elseif mod(mx(l),2) == 1 && mod(my(l),2)==0
                    Ex(l,inc_lam,inc1+1:inc1+inc2,xspread,yspread) = facx(l).*fbx(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1xx(l,ll).*norm1xy(l,ll).*sin(by(l,ll).*Y2).*sin(bx(l,ll).*X2));
                    Ey(l,inc_lam,inc1+1:inc1+inc2,xspread,yspread) = facy(l).*fby(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1yx(l,ll).*norm1yy(l,ll).*cos(by(l,ll).*Y2).*cos(bx(l,ll).*X2));
                    if l > nb_modes_TE
                        Ez(l,inc_lam,inc1+1:inc1+inc2,xspread,yspread) = fac1x(l).*fac2y(l).*1i.*sqrt(bx(l,ll).^2+by(l,ll).^2)./beta(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1xy(l,ll).*norm1yx(l,ll).*sin(by(l,ll).*Y2).*cos(bx(l,ll).*X2));
                    end
                else
                    Ex(l,inc_lam,inc1+1:inc1+inc2,xspread,yspread) =facx(l).*fbx(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1xx(l,ll).*norm1xy(l,ll).*sin(by(l,ll).*Y2).*cos(bx(l,ll).*X2));
                    Ey(l,inc_lam,inc1+1:inc1+inc2,xspread,yspread) =facy(l).*fby(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1yx(l,ll).*norm1yy(l,ll).*cos(by(l,ll).*Y2).*sin(bx(l,ll).*X2));
                    if l > nb_modes_TE
                        Ez(l,inc_lam,inc1+1:inc1+inc2,xspread,yspread) = fac1x(l).*fac2y(l).*1i.*sqrt(bx(l,ll).^2+by(l,ll).^2)./beta(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z2)+B(l,ll).*exp(-1i.*beta(l,ll).*Z2)).*(norm1xy(l,ll).*norm1yx(l,ll).*sin(by(l,ll).*Y2).*sin(bx(l,ll).*X2));
                    end
                end
            end
        end
    end
    
else
    z = plane;
    if plane <= -h/2 || plane >= h/2
        x = -d/2:step:d/2;
        y = -d/2:step:d/2;
    else
        x = -a/2:step:a/2;
        y = -a/2:step:a/2;
    end
    
    inc = 0;
    inc1 = 0;inc2 = 0;inc3 = 0;inc4 = 0;
    [X,Z,Y] = meshgrid(x,z,y);
    %Hy = complex(zeros(nb_difftot*2,1,length(z),length(x),length(y)));
    Ex = complex(zeros(nb_difftot2,1,length(z),length(x),length(y)));
    Ez = complex(zeros(nb_difftot2,1,length(z),length(x),length(y)));
    Ey = complex(zeros(nb_difftot2,1,length(z),length(x),length(y)));
    
    for ll = y_p
        inc_lam = 1;
        for l2 = 1:nb_difftot2
            crit_l = floor((l2-1)./nb_difftot);
            l =  mod(l2-1,nb_difftot)+1;
            if crit_l ==0
                if l == n0S
                    if plane <= -h/2
                        Ex(l,inc_lam,:,:,:) = fx1(l,ll)./d.*(exp(1i.*kx(l,ll).*(X)).*exp(1i.*ky(l,ll).*(Y)).*exp(1i.*kz(l,ll).*(Z+h/2))+ refl(l,ll).*exp(1i.*kx(l,ll).*(X)).*exp(1i.*ky(l,ll).*(Y)).*exp(-1i.*kz(l,ll).*(Z+h/2)));
                        Ey(l,inc_lam,:,:,:) = fy1(l,ll)./d.*(exp(1i.*kx(l,ll).*(X)).*exp(1i.*ky(l,ll).*(Y)).*exp(1i.*kz(l,ll).*(Z+h/2))+ refl(l,ll).*exp(1i.*kx(l,ll).*(X)).*exp(1i.*ky(l,ll).*(Y)).*exp(-1i.*kz(l,ll).*(Z+h/2)));
                        Ez(l,inc_lam,:,:,:) = -kll(l,ll)./kz(l,ll)./d.*(exp(1i.*kx(l,ll).*(X)).*exp(1i.*ky(l,ll).*(Y)).*exp(1i.*kz(l,ll).*(Z+h/2))-refl(l,ll).*exp(1i.*kx(l,ll).*(X)).*exp(1i.*ky(l,ll).*(Y)).*exp(-1i.*kz(l,ll).*(Z+h/2)));
                    elseif plane >= h/2
                        Ex(l,inc_lam,:,:,:) = fx3(l,ll)./d.*(1.*trans(l,ll).*exp(1i.*kx3(l,ll).*(X)).*exp(1i.*ky3(l,ll).*(Y)).*exp(1i.*kz3(l,ll).*(Z-h/2)));
                        Ey(l,inc_lam,:,:,:) = fy3(l,ll)./d.*(1.*trans(l,ll).*exp(1i.*kx3(l,ll).*(X)).*exp(1i.*ky3(l,ll).*(Y)).*exp(1i.*kz3(l,ll).*(Z-h/2)));
                        Ez(l,inc_lam,:,:,:) = kll3(l,ll)./kz3(l,ll)./d.*(1.*trans(l,ll).*exp(1i.*kx3(l,ll).*(X)).*exp(1i.*ky3(l,ll).*(Y)).*exp(1i.*kz3(l,ll).*(Z-h/2)));
                    end
                else
                    if plane <= -h/2
                        Ex(l,inc_lam,:,:,:) = fx1(l,ll)./d.*(refl(l,ll).*exp(1i.*kx(l,ll).*(X)).*exp(1i.*ky(l,ll).*(Y)).*exp(-1i.*kz(l,ll).*(Z+h/2)));
                        Ey(l,inc_lam,:,:,:) = fy1(l,ll)./d.*(refl(l,ll).*exp(1i.*kx(l,ll).*(X)).*exp(1i.*ky(l,ll).*(Y)).*exp(-1i.*kz(l,ll).*(Z+h/2)));
                        Ez(l,inc_lam,:,:,:) = kll(l,ll)./kz(l,ll)./d.*(refl(l,ll).*exp(1i.*kx(l,ll).*(X)).*exp(1i.*ky(l,ll).*(Y)).*exp(-1i.*kz(l,ll).*(Z+h/2)));
                    elseif plane >= h/2
                        Ex(l,inc_lam,:,:,:) = fx3(l,ll)./d.*(1.*trans(l,ll).*exp(1i.*kx3(l,ll).*(X)).*exp(1i.*ky3(l,ll).*(Y)).*exp(1i.*kz3(l,ll).*(Z-h/2)));
                        Ey(l,inc_lam,:,:,:) = fy3(l,ll)./d.*(1.*trans(l,ll).*exp(1i.*kx3(l,ll).*(X)).*exp(1i.*ky3(l,ll).*(Y)).*exp(1i.*kz3(l,ll).*(Z-h/2)));
                        Ez(l,inc_lam,:,:,:) = kll3(l,ll)./kz(l,ll)./d.*(trans(l,ll).*exp(1i.*kx3(l,ll).*(X)).*exp(1i.*ky3(l,ll).*(Y)).*exp(1i.*kz3(l,ll).*(Z-h/2)));
                    end
                end
            else
                if plane <= -h/2
                    Ex(l+nb_difftot,inc_lam,:,:,:) = fx1(nb_difftot+l,ll)./d.*(refl(l+nb_difftot,ll).*exp(1i.*kx(l,ll).*(X)).*exp(1i.*ky(l,ll).*(Y)).*exp(-1i.*kz(l,ll).*(Z+h/2)));
                    Ey(l+nb_difftot,inc_lam,:,:,:) = fy1(nb_difftot+l,ll)./d.*(refl(l+nb_difftot,ll).*exp(1i.*kx(l,ll).*(X)).*exp(1i.*ky(l,ll).*(Y)).*exp(-1i.*kz(l,ll).*(Z+h/2)));
                    Ez(l+nb_difftot,inc_lam,:,:,:) = 0./kz(l,ll)./d.*(refl(l+nb_difftot,ll).*exp(1i.*kx(l,ll).*(X)).*exp(1i.*ky(l,ll).*(Y)).*exp(-1i.*kz(l,ll).*(Z+h/2)));
                elseif plane >= h/2
                    Ex(l+nb_difftot,inc_lam,:,:,:) = fx3(nb_difftot+l,ll)./d.*(1.*trans(l+nb_difftot,ll).*exp(1i.*kx3(l,ll).*(X)).*exp(1i.*ky3(l,ll).*(Y)).*exp(1i.*kz3(l,ll).*(Z-h/2)));
                    Ey(l+nb_difftot,inc_lam,:,:,:) = fy3(nb_difftot+l,ll)./d.*(1.*trans(l+nb_difftot,ll).*exp(1i.*kx3(l,ll).*(X)).*exp(1i.*ky3(l,ll).*(Y)).*exp(1i.*kz3(l,ll).*(Z-h/2)));
                    Ez(l+nb_difftot,inc_lam,:,:,:) = -0./kz(l,ll)./d.*(trans(l+nb_difftot,ll).*exp(1i.*kx3(l,ll).*(X)).*exp(1i.*ky3(l,ll).*(Y)).*exp(1i.*kz3(l,ll).*(Z-h/2)));
                end
            end
        end
        for l = 1:nb_modes
            if plane < h/2 && plane > -h/2
                if mod(mx(l),2) == 0 && mod(my(l),2)==1
                    Ex(l,inc_lam,:,:,:) = facx(l).*fbx(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z)+B(l,ll).*exp(-1i.*beta(l,ll).*Z)).*(norm1xx(l,ll).*norm1xy(l,ll).*cos(by(l,ll).*Y).*cos(bx(l,ll).*X));
                    Ey(l,inc_lam,:,:,:) = facy(l).*fby(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z)+B(l,ll).*exp(-1i.*beta(l,ll).*Z)).*(norm1yx(l,ll).*norm1yy(l,ll).*sin(by(l,ll).*Y).*sin(bx(l,ll).*X));
                    if l > nb_modes_TE
                        Ez(l,inc_lam,:,:,:) = fac1x(l).*fac2y(l).*1i.*sqrt(bx(l,ll).^2+by(l,ll).^2)./beta(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z)+B(l,ll).*exp(-1i.*beta(l,ll).*Z)).*(norm1xy(l,ll).*norm1yx(l,ll).*cos(by(l,ll).*Y).*sin(bx(l,ll).*X));
                    end
                elseif mod(mx(l),2) == 1 && mod(my(l),2)==1
                    Ex(l,inc_lam,:,:,:) = facx(l).*fbx(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z)+B(l,ll).*exp(-1i.*beta(l,ll).*Z)).*(norm1xx(l,ll).*norm1xy(l,ll).*cos(by(l,ll).*Y).*sin(bx(l,ll).*X));
                    Ey(l,inc_lam,:,:,:) = facy(l).*fby(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z)+B(l,ll).*exp(-1i.*beta(l,ll).*Z)).*(norm1yx(l,ll).*norm1yy(l,ll).*sin(by(l,ll).*Y).*cos(bx(l,ll).*X));
                    if l > nb_modes_TE
                        Ez(l,inc_lam,:,:,:) = fac1x(l).*fac2y(l).*1i.*sqrt(bx(l,ll).^2+by(l,ll).^2)./beta(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z)+B(l,ll).*exp(-1i.*beta(l,ll).*Z)).*(norm1xy(l,ll).*norm1yx(l,ll).*cos(by(l,ll).*Y).*cos(bx(l,ll).*X));
                    end
                elseif mod(mx(l),2) == 1 && mod(my(l),2)==0
                    Ex(l,inc_lam,:,:,:) = facx(l).*fbx(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z)+B(l,ll).*exp(-1i.*beta(l,ll).*Z)).*(norm1xx(l,ll).*norm1xy(l,ll).*sin(by(l,ll).*Y).*sin(bx(l,ll).*X));
                    Ey(l,inc_lam,:,:,:) = facy(l).*fby(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z)+B(l,ll).*exp(-1i.*beta(l,ll).*Z)).*(norm1yx(l,ll).*norm1yy(l,ll).*cos(by(l,ll).*Y).*cos(bx(l,ll).*X));
                    if l > nb_modes_TE
                        Ez(l,inc_lam,:,:,:) = fac1x(l).*fac2y(l).*1i.*sqrt(bx(l,ll).^2+by(l,ll).^2)./beta(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z)+B(l,ll).*exp(-1i.*beta(l,ll).*Z)).*(norm1xy(l,ll).*norm1yx(l,ll).*sin(by(l,ll).*Y).*cos(bx(l,ll).*X));
                    end
                else
                    Ex(l,inc_lam,:,:,:) =facx(l).*fbx(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z)+B(l,ll).*exp(-1i.*beta(l,ll).*Z)).*(norm1xx(l,ll).*norm1xy(l,ll).*sin(by(l,ll).*Y).*cos(bx(l,ll).*X));
                    Ey(l,inc_lam,:,:,:) =facy(l).*fby(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z)+B(l,ll).*exp(-1i.*beta(l,ll).*Z)).*(norm1yx(l,ll).*norm1yy(l,ll).*cos(by(l,ll).*Y).*sin(bx(l,ll).*X));
                    if l > nb_modes_TE
                        Ez(l,inc_lam,:,:,:) = fac1x(l).*fac2y(l).*1i.*sqrt(bx(l,ll).^2+by(l,ll).^2)./beta(l,ll).*(A(l,1).*exp(1i.*beta(l,ll).*Z)+B(l,ll).*exp(-1i.*beta(l,ll).*Z)).*(norm1xy(l,ll).*norm1yx(l,ll).*sin(by(l,ll).*Y).*sin(bx(l,ll).*X));
                    end
                end
            end
        end
    end
end

E =(squeeze(abs(sum(Ex(:,1,:,:,:),1)).^2)+squeeze(abs(sum(Ey(:,1,:,:,:),1)).^2)+squeeze(abs(sum(Ez(:,1,:,:,:),1)).^2));

if pp==1
    absc = x;
    ord = z;
elseif pp==2
    absc = y;
    ord = z;
else
    absc = x;
    ord = y;
end

if length(find(imag(beta)<0))>1
    fprintf(2,'Warning, imag(beta) is negative, fminsearch wasn''t effective. Use a smaller lambda as first value \n')
end
transn0=trans(n0S,:);refln0=refl(n0S,:);
